from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .market_order_delayed_trade_close import MarketOrderDelayedTradeClose
from .market_order_margin_closeout import MarketOrderMarginCloseout
from .market_order_position_closeout import MarketOrderPositionCloseout
from .market_order_position_fill import MarketOrderPositionFill
from .market_order_position_fill import check_market_order_position_fill
from .market_order_state import MarketOrderState
from .market_order_state import check_market_order_state
from .market_order_time_in_force import MarketOrderTimeInForce
from .market_order_time_in_force import check_market_order_time_in_force
from .market_order_trade_close import MarketOrderTradeClose
from .market_order_type import MarketOrderType
from .market_order_type import check_market_order_type
from .stop_loss_details import StopLossDetails
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="MarketOrder")


@dataclasses.dataclass
class MarketOrder:
    """A MarketOrder is an order that is filled immediately upon creation using the current market price.

    Attributes:
        id (Optional[str]): The Order's identifier, unique within the Order's Account.
        create_time (Optional[str]): The time when the Order was created.
        state (Optional[MarketOrderState]): The current state of the Order.
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        type (Optional[MarketOrderType]): The type of the Order. Always set to "MARKET" for Market Orders.
        instrument (Optional[str]): The Market Order's Instrument.
        units (Optional[str]): The quantity requested to be filled by the Market Order. A posititive number of units
            results in a long Order, and a negative number of units results in a short Order.
        time_in_force (Optional[MarketOrderTimeInForce]): The time-in-force requested for the Market Order.
            Restricted to FOK or IOC for a MarketOrder.
        price_bound (Optional[str]): The worst price that the client is willing to have the Market Order filled at.
        position_fill (Optional[MarketOrderPositionFill]): Specification of how Positions in the Account are
            modified when the Order is filled.
        trade_close (Optional[MarketOrderTradeClose]): A MarketOrderTradeClose specifies the extensions to a Market
            Order that has been created specifically to close a Trade.
        long_position_closeout (Optional[MarketOrderPositionCloseout]): A MarketOrderPositionCloseout specifies the
            extensions to a Market Order when it has been created to closeout a specific Position.
        short_position_closeout (Optional[MarketOrderPositionCloseout]): A MarketOrderPositionCloseout specifies the
            extensions to a Market Order when it has been created to closeout a specific Position.
        margin_closeout (Optional[MarketOrderMarginCloseout]): Details for the Market Order extensions specific to a
            Market Order placed that is part of a Market Order Margin Closeout in a client's account
        delayed_trade_close (Optional[MarketOrderDelayedTradeClose]): Details for the Market Order extensions
            specific to a Market Order placed with the intent of fully closing a specific open trade that should have
            already been closed but wasn't due to halted market conditions
        take_profit_on_fill (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        filling_transaction_id (Optional[str]): ID of the Transaction that filled this Order (only provided when the
            Order's state is FILLED)
        filled_time (Optional[str]): Date/time when the Order was filled (only provided when the Order's state is
            FILLED)
        trade_opened_id (Optional[str]): Trade ID of Trade opened when the Order was filled (only provided when the
            Order's state is FILLED and a Trade was opened as a result of the fill)
        trade_reduced_id (Optional[str]): Trade ID of Trade reduced when the Order was filled (only provided when
            the Order's state is FILLED and a Trade was reduced as a result of the fill)
        trade_closed_i_ds (Optional[List[str]]): Trade IDs of Trades closed when the Order was filled (only provided
            when the Order's state is FILLED and one or more Trades were closed as a result of the fill)
        cancelling_transaction_id (Optional[str]): ID of the Transaction that cancelled the Order (only provided
            when the Order's state is CANCELLED)
        cancelled_time (Optional[str]): Date/time when the Order was cancelled (only provided when the state of the
            Order is CANCELLED)"""

    id: Optional[str]
    create_time: Optional[str]
    state: Optional[MarketOrderState]
    client_extensions: Optional["ClientExtensions"]
    type: Optional[MarketOrderType]
    instrument: Optional[str]
    units: Optional[str]
    time_in_force: Optional[MarketOrderTimeInForce]
    price_bound: Optional[str]
    position_fill: Optional[MarketOrderPositionFill]
    trade_close: Optional["MarketOrderTradeClose"]
    long_position_closeout: Optional["MarketOrderPositionCloseout"]
    short_position_closeout: Optional["MarketOrderPositionCloseout"]
    margin_closeout: Optional["MarketOrderMarginCloseout"]
    delayed_trade_close: Optional["MarketOrderDelayedTradeClose"]
    take_profit_on_fill: Optional["TakeProfitDetails"]
    stop_loss_on_fill: Optional["StopLossDetails"]
    trailing_stop_loss_on_fill: Optional["TrailingStopLossDetails"]
    trade_client_extensions: Optional["ClientExtensions"]
    filling_transaction_id: Optional[str]
    filled_time: Optional[str]
    trade_opened_id: Optional[str]
    trade_reduced_id: Optional[str]
    trade_closed_i_ds: Optional[List[str]]
    cancelling_transaction_id: Optional[str]
    cancelled_time: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .market_order_position_closeout import MarketOrderPositionCloseout
        from .client_extensions import ClientExtensions
        from .take_profit_details import TakeProfitDetails
        from .market_order_margin_closeout import MarketOrderMarginCloseout
        from .market_order_delayed_trade_close import MarketOrderDelayedTradeClose
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .stop_loss_details import StopLossDetails
        from .market_order_trade_close import MarketOrderTradeClose

        d = src_dict.copy()
        id = d.pop("id", None)
        create_time = d.pop("createTime", None)
        _state = d.pop("state", None)
        state: Optional[MarketOrderState]
        if _state is None:
            state = None
        else:
            state = check_market_order_state(_state)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _type = d.pop("type", None)
        type: Optional[MarketOrderType]
        if _type is None:
            type = None
        else:
            type = check_market_order_type(_type)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[MarketOrderTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_market_order_time_in_force(_time_in_force)
        price_bound = d.pop("priceBound", None)
        _position_fill = d.pop("positionFill", None)
        position_fill: Optional[MarketOrderPositionFill]
        if _position_fill is None:
            position_fill = None
        else:
            position_fill = check_market_order_position_fill(_position_fill)
        _trade_close = d.pop("tradeClose", None)
        trade_close: Optional[MarketOrderTradeClose]
        if _trade_close is None:
            trade_close = None
        else:
            trade_close = MarketOrderTradeClose.from_dict(_trade_close)
        _long_position_closeout = d.pop("longPositionCloseout", None)
        long_position_closeout: Optional[MarketOrderPositionCloseout]
        if _long_position_closeout is None:
            long_position_closeout = None
        else:
            long_position_closeout = MarketOrderPositionCloseout.from_dict(
                _long_position_closeout
            )
        _short_position_closeout = d.pop("shortPositionCloseout", None)
        short_position_closeout: Optional[MarketOrderPositionCloseout]
        if _short_position_closeout is None:
            short_position_closeout = None
        else:
            short_position_closeout = MarketOrderPositionCloseout.from_dict(
                _short_position_closeout
            )
        _margin_closeout = d.pop("marginCloseout", None)
        margin_closeout: Optional[MarketOrderMarginCloseout]
        if _margin_closeout is None:
            margin_closeout = None
        else:
            margin_closeout = MarketOrderMarginCloseout.from_dict(_margin_closeout)
        _delayed_trade_close = d.pop("delayedTradeClose", None)
        delayed_trade_close: Optional[MarketOrderDelayedTradeClose]
        if _delayed_trade_close is None:
            delayed_trade_close = None
        else:
            delayed_trade_close = MarketOrderDelayedTradeClose.from_dict(
                _delayed_trade_close
            )
        _take_profit_on_fill = d.pop("takeProfitOnFill", None)
        take_profit_on_fill: Optional[TakeProfitDetails]
        if _take_profit_on_fill is None:
            take_profit_on_fill = None
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)
        _stop_loss_on_fill = d.pop("stopLossOnFill", None)
        stop_loss_on_fill: Optional[StopLossDetails]
        if _stop_loss_on_fill is None:
            stop_loss_on_fill = None
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)
        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", None)
        trailing_stop_loss_on_fill: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss_on_fill is None:
            trailing_stop_loss_on_fill = None
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        filling_transaction_id = d.pop("fillingTransactionID", None)
        filled_time = d.pop("filledTime", None)
        trade_opened_id = d.pop("tradeOpenedID", None)
        trade_reduced_id = d.pop("tradeReducedID", None)
        trade_closed_i_ds = cast(List[str], d.pop("tradeClosedIDs", None))
        cancelling_transaction_id = d.pop("cancellingTransactionID", None)
        cancelled_time = d.pop("cancelledTime", None)
        market_order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
            type=type,
            instrument=instrument,
            units=units,
            time_in_force=time_in_force,
            price_bound=price_bound,
            position_fill=position_fill,
            trade_close=trade_close,
            long_position_closeout=long_position_closeout,
            short_position_closeout=short_position_closeout,
            margin_closeout=margin_closeout,
            delayed_trade_close=delayed_trade_close,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
            filling_transaction_id=filling_transaction_id,
            filled_time=filled_time,
            trade_opened_id=trade_opened_id,
            trade_reduced_id=trade_reduced_id,
            trade_closed_i_ds=trade_closed_i_ds,
            cancelling_transaction_id=cancelling_transaction_id,
            cancelled_time=cancelled_time,
        )
        market_order.additional_properties = d
        return market_order

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .trailing_stop_loss_order_state import TrailingStopLossOrderState
from .trailing_stop_loss_order_state import check_trailing_stop_loss_order_state
from .trailing_stop_loss_order_time_in_force import TrailingStopLossOrderTimeInForce
from .trailing_stop_loss_order_time_in_force import (
    check_trailing_stop_loss_order_time_in_force,
)
from .trailing_stop_loss_order_trigger_condition import (
    TrailingStopLossOrderTriggerCondition,
)
from .trailing_stop_loss_order_trigger_condition import (
    check_trailing_stop_loss_order_trigger_condition,
)
from .trailing_stop_loss_order_type import TrailingStopLossOrderType
from .trailing_stop_loss_order_type import check_trailing_stop_loss_order_type
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="TrailingStopLossOrder")


@dataclasses.dataclass
class TrailingStopLossOrder:
    """A TrailingStopLossOrder is an order that is linked to an open Trade and created with a price distance. The price
    distance is used to calculate a trailing stop value for the order that is in the losing direction from the market
    price at the time of the order's creation. The trailing stop value will follow the market price as it moves in the
    winning direction, and the order will filled (closing the Trade) by the first price that is equal to or worse than
    the trailing stop value. A TrailingStopLossOrder cannot be used to open a new Position.

        Attributes:
            id (Optional[str]): The Order's identifier, unique within the Order's Account.
            create_time (Optional[str]): The time when the Order was created.
            state (Optional[TrailingStopLossOrderState]): The current state of the Order.
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            type (Optional[TrailingStopLossOrderType]): The type of the Order. Always set to "TRAILING_STOP_LOSS" for
                Trailing Stop Loss Orders.
            trade_id (Optional[str]): The ID of the Trade to close when the price threshold is breached.
            client_trade_id (Optional[str]): The client ID of the Trade to be closed when the price threshold is
                breached.
            distance (Optional[str]): The price distance (in price units) specified for the TrailingStopLoss Order.
            time_in_force (Optional[TrailingStopLossOrderTimeInForce]): The time-in-force requested for the
                TrailingStopLoss Order. Restricted to "GTC", "GFD" and "GTD" for TrailingStopLoss Orders.
            gtd_time (Optional[str]): The date/time when the StopLoss Order will be cancelled if its timeInForce is
                "GTD".
            trigger_condition (Optional[TrailingStopLossOrderTriggerCondition]): Specification of which price component
                should be used when determining if an Order should be triggered and filled. This allows Orders to be triggered
                based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy) price
                depending on the desired behaviour. Orders are always filled using their default price component.
                This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
                condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
                transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
                is set to the default value when indicating the distance from an Order's trigger price, and will always provide
                the default trigger condition when creating or modifying an Order.
                A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
                value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
                a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
            trailing_stop_value (Optional[str]): The trigger price for the Trailing Stop Loss Order. The trailing stop
                value will trail (follow) the market price by the TSL order's configured "distance" as the market price moves in
                the winning direction. If the market price moves to a level that is equal to or worse than the trailing stop
                value, the order will be filled and the Trade will be closed.
            filling_transaction_id (Optional[str]): ID of the Transaction that filled this Order (only provided when the
                Order's state is FILLED)
            filled_time (Optional[str]): Date/time when the Order was filled (only provided when the Order's state is
                FILLED)
            trade_opened_id (Optional[str]): Trade ID of Trade opened when the Order was filled (only provided when the
                Order's state is FILLED and a Trade was opened as a result of the fill)
            trade_reduced_id (Optional[str]): Trade ID of Trade reduced when the Order was filled (only provided when
                the Order's state is FILLED and a Trade was reduced as a result of the fill)
            trade_closed_i_ds (Optional[List[str]]): Trade IDs of Trades closed when the Order was filled (only provided
                when the Order's state is FILLED and one or more Trades were closed as a result of the fill)
            cancelling_transaction_id (Optional[str]): ID of the Transaction that cancelled the Order (only provided
                when the Order's state is CANCELLED)
            cancelled_time (Optional[str]): Date/time when the Order was cancelled (only provided when the state of the
                Order is CANCELLED)
            replaces_order_id (Optional[str]): The ID of the Order that was replaced by this Order (only provided if
                this Order was created as part of a cancel/replace).
            replaced_by_order_id (Optional[str]): The ID of the Order that replaced this Order (only provided if this
                Order was cancelled as part of a cancel/replace)."""

    id: Optional[str]
    create_time: Optional[str]
    state: Optional[TrailingStopLossOrderState]
    client_extensions: Optional["ClientExtensions"]
    type: Optional[TrailingStopLossOrderType]
    trade_id: Optional[str]
    client_trade_id: Optional[str]
    distance: Optional[str]
    time_in_force: Optional[TrailingStopLossOrderTimeInForce]
    gtd_time: Optional[str]
    trigger_condition: Optional[TrailingStopLossOrderTriggerCondition]
    trailing_stop_value: Optional[str]
    filling_transaction_id: Optional[str]
    filled_time: Optional[str]
    trade_opened_id: Optional[str]
    trade_reduced_id: Optional[str]
    trade_closed_i_ds: Optional[List[str]]
    cancelling_transaction_id: Optional[str]
    cancelled_time: Optional[str]
    replaces_order_id: Optional[str]
    replaced_by_order_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        id = d.pop("id", None)
        create_time = d.pop("createTime", None)
        _state = d.pop("state", None)
        state: Optional[TrailingStopLossOrderState]
        if _state is None:
            state = None
        else:
            state = check_trailing_stop_loss_order_state(_state)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _type = d.pop("type", None)
        type: Optional[TrailingStopLossOrderType]
        if _type is None:
            type = None
        else:
            type = check_trailing_stop_loss_order_type(_type)
        trade_id = d.pop("tradeID", None)
        client_trade_id = d.pop("clientTradeID", None)
        distance = d.pop("distance", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[TrailingStopLossOrderTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_trailing_stop_loss_order_time_in_force(_time_in_force)
        gtd_time = d.pop("gtdTime", None)
        _trigger_condition = d.pop("triggerCondition", None)
        trigger_condition: Optional[TrailingStopLossOrderTriggerCondition]
        if _trigger_condition is None:
            trigger_condition = None
        else:
            trigger_condition = check_trailing_stop_loss_order_trigger_condition(
                _trigger_condition
            )
        trailing_stop_value = d.pop("trailingStopValue", None)
        filling_transaction_id = d.pop("fillingTransactionID", None)
        filled_time = d.pop("filledTime", None)
        trade_opened_id = d.pop("tradeOpenedID", None)
        trade_reduced_id = d.pop("tradeReducedID", None)
        trade_closed_i_ds = cast(List[str], d.pop("tradeClosedIDs", None))
        cancelling_transaction_id = d.pop("cancellingTransactionID", None)
        cancelled_time = d.pop("cancelledTime", None)
        replaces_order_id = d.pop("replacesOrderID", None)
        replaced_by_order_id = d.pop("replacedByOrderID", None)
        trailing_stop_loss_order = cls(
            id=id,
            create_time=create_time,
            state=state,
            client_extensions=client_extensions,
            type=type,
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            distance=distance,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            trigger_condition=trigger_condition,
            trailing_stop_value=trailing_stop_value,
            filling_transaction_id=filling_transaction_id,
            filled_time=filled_time,
            trade_opened_id=trade_opened_id,
            trade_reduced_id=trade_reduced_id,
            trade_closed_i_ds=trade_closed_i_ds,
            cancelling_transaction_id=cancelling_transaction_id,
            cancelled_time=cancelled_time,
            replaces_order_id=replaces_order_id,
            replaced_by_order_id=replaced_by_order_id,
        )
        trailing_stop_loss_order.additional_properties = d
        return trailing_stop_loss_order

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .trailing_stop_loss_order_request_time_in_force import (
    TrailingStopLossOrderRequestTimeInForce,
)
from .trailing_stop_loss_order_request_time_in_force import (
    check_trailing_stop_loss_order_request_time_in_force,
)
from .trailing_stop_loss_order_request_trigger_condition import (
    TrailingStopLossOrderRequestTriggerCondition,
)
from .trailing_stop_loss_order_request_trigger_condition import (
    check_trailing_stop_loss_order_request_trigger_condition,
)
from .trailing_stop_loss_order_request_type import TrailingStopLossOrderRequestType
from .trailing_stop_loss_order_request_type import (
    check_trailing_stop_loss_order_request_type,
)
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="TrailingStopLossOrderRequest")


@dataclasses.dataclass
class TrailingStopLossOrderRequest:
    """A TrailingStopLossOrderRequest specifies the parameters that may be set when creating a Trailing Stop Loss Order.

    Attributes:
        type (Optional[TrailingStopLossOrderRequestType]): The type of the Order to Create. Must be set to
            "TRAILING_STOP_LOSS" when creating a Trailng Stop Loss Order.
        trade_id (Optional[str]): The ID of the Trade to close when the price threshold is breached.
        client_trade_id (Optional[str]): The client ID of the Trade to be closed when the price threshold is
            breached.
        distance (Optional[str]): The price distance (in price units) specified for the TrailingStopLoss Order.
        time_in_force (Optional[TrailingStopLossOrderRequestTimeInForce]): The time-in-force requested for the
            TrailingStopLoss Order. Restricted to "GTC", "GFD" and "GTD" for TrailingStopLoss Orders.
        gtd_time (Optional[str]): The date/time when the StopLoss Order will be cancelled if its timeInForce is
            "GTD".
        trigger_condition (Optional[TrailingStopLossOrderRequestTriggerCondition]): Specification of which price
            component should be used when determining if an Order should be triggered and filled. This allows Orders to be
            triggered based on the bid, ask, mid, default (ask for buy, bid for sell) or inverse (ask for sell, bid for buy)
            price depending on the desired behaviour. Orders are always filled using their default price component.
            This feature is only provided through the REST API. Clients who choose to specify a non-default trigger
            condition will not see it reflected in any of OANDA's proprietary or partner trading platforms, their
            transaction history or their account statements. OANDA platforms always assume that an Order's trigger condition
            is set to the default value when indicating the distance from an Order's trigger price, and will always provide
            the default trigger condition when creating or modifying an Order.
            A special restriction applies when creating a guaranteed Stop Loss Order. In this case the TriggerCondition
            value must either be "DEFAULT", or the "natural" trigger side "DEFAULT" results in. So for a Stop Loss Order for
            a long trade valid values are "DEFAULT" and "BID", and for short trades "DEFAULT" and "ASK" are valid.
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4."""

    type: Optional[TrailingStopLossOrderRequestType]
    trade_id: Optional[str]
    client_trade_id: Optional[str]
    distance: Optional[str]
    time_in_force: Optional[TrailingStopLossOrderRequestTimeInForce]
    gtd_time: Optional[str]
    trigger_condition: Optional[TrailingStopLossOrderRequestTriggerCondition]
    client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .client_extensions import ClientExtensions

        d = src_dict.copy()
        _type = d.pop("type", None)
        type: Optional[TrailingStopLossOrderRequestType]
        if _type is None:
            type = None
        else:
            type = check_trailing_stop_loss_order_request_type(_type)
        trade_id = d.pop("tradeID", None)
        client_trade_id = d.pop("clientTradeID", None)
        distance = d.pop("distance", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[TrailingStopLossOrderRequestTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_trailing_stop_loss_order_request_time_in_force(
                _time_in_force
            )
        gtd_time = d.pop("gtdTime", None)
        _trigger_condition = d.pop("triggerCondition", None)
        trigger_condition: Optional[TrailingStopLossOrderRequestTriggerCondition]
        if _trigger_condition is None:
            trigger_condition = None
        else:
            trigger_condition = (
                check_trailing_stop_loss_order_request_trigger_condition(
                    _trigger_condition
                )
            )
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        trailing_stop_loss_order_request = cls(
            type=type,
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            distance=distance,
            time_in_force=time_in_force,
            gtd_time=gtd_time,
            trigger_condition=trigger_condition,
            client_extensions=client_extensions,
        )
        trailing_stop_loss_order_request.additional_properties = d
        return trailing_stop_loss_order_request

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

GetInstrumentCandlesResponse200Granularity = Literal[
    "D",
    "H1",
    "H12",
    "H2",
    "H3",
    "H4",
    "H6",
    "H8",
    "M",
    "M1",
    "M10",
    "M15",
    "M2",
    "M30",
    "M4",
    "M5",
    "S10",
    "S15",
    "S30",
    "S5",
    "W",
]
GET_INSTRUMENT_CANDLES_RESPONSE_200_GRANULARITY_VALUES: Set[
    GetInstrumentCandlesResponse200Granularity
] = {
    "D",
    "H1",
    "H12",
    "H2",
    "H3",
    "H4",
    "H6",
    "H8",
    "M",
    "M1",
    "M10",
    "M15",
    "M2",
    "M30",
    "M4",
    "M5",
    "S10",
    "S15",
    "S30",
    "S5",
    "W",
}


def check_get_instrument_candles_response_200_granularity(
    value: str,
) -> GetInstrumentCandlesResponse200Granularity:
    if value in GET_INSTRUMENT_CANDLES_RESPONSE_200_GRANULARITY_VALUES:
        return cast(GetInstrumentCandlesResponse200Granularity, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {GET_INSTRUMENT_CANDLES_RESPONSE_200_GRANULARITY_VALUES!r}"
    )



from __future__ import annotations
from typing import Literal, Set, cast

TradePL = Literal["NEGATIVE", "POSITIVE", "ZERO"]
TRADE_PL_VALUES: Set[TradePL] = {"NEGATIVE", "POSITIVE", "ZERO"}


def check_trade_pl(value: str) -> TradePL:
    if value in TRADE_PL_VALUES:
        return cast(TradePL, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {TRADE_PL_VALUES!r}")



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.list_transactions_response_200_type_item import (
    check_list_transactions_response_200_type_item,
)
from ..models.list_transactions_response_200_type_item import (
    ListTransactionsResponse200TypeItem,
)
from typing import cast
from typing import Union


T = TypeVar("T", bound="ListTransactionsResponse200")


@_attrs_define
class ListTransactionsResponse200:
    """
    Attributes:
        from_ (Union[Unset, str]): The starting time provided in the request.
        to (Union[Unset, str]): The ending time provided in the request.
        page_size (Union[Unset, int]): The pageSize provided in the request
        type (Union[Unset, List[ListTransactionsResponse200TypeItem]]): The Transaction-type filter provided in the
            request
        count (Union[Unset, int]): The number of Transactions that are contained in the pages returned
        pages (Union[Unset, List[str]]): The list of URLs that represent idrange queries providing the data for each
            page in the query results
        last_transaction_id (Union[Unset, str]): The ID of the most recent Transaction created for the Account
    """

    from_: Union[Unset, str] = UNSET
    to: Union[Unset, str] = UNSET
    page_size: Union[Unset, int] = UNSET
    type: Union[Unset, List[ListTransactionsResponse200TypeItem]] = UNSET
    count: Union[Unset, int] = UNSET
    pages: Union[Unset, List[str]] = UNSET
    last_transaction_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from_ = self.from_

        to = self.to

        page_size = self.page_size

        type: Union[Unset, List[str]] = UNSET
        if not isinstance(self.type, Unset):
            type = []
            for type_item_data in self.type:
                type_item: str = type_item_data
                type.append(type_item)

        count = self.count

        pages: Union[Unset, List[str]] = UNSET
        if not isinstance(self.pages, Unset):
            pages = self.pages

        last_transaction_id = self.last_transaction_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if from_ is not UNSET:
            field_dict["from"] = from_
        if to is not UNSET:
            field_dict["to"] = to
        if page_size is not UNSET:
            field_dict["pageSize"] = page_size
        if type is not UNSET:
            field_dict["type"] = type
        if count is not UNSET:
            field_dict["count"] = count
        if pages is not UNSET:
            field_dict["pages"] = pages
        if last_transaction_id is not UNSET:
            field_dict["lastTransactionID"] = last_transaction_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        from_ = d.pop("from", UNSET)

        to = d.pop("to", UNSET)

        page_size = d.pop("pageSize", UNSET)

        type = []
        _type = d.pop("type", UNSET)
        for type_item_data in _type or []:
            type_item = check_list_transactions_response_200_type_item(type_item_data)

            type.append(type_item)

        count = d.pop("count", UNSET)

        pages = cast(List[str], d.pop("pages", UNSET))

        last_transaction_id = d.pop("lastTransactionID", UNSET)

        list_transactions_response_200 = cls(
            from_=from_,
            to=to,
            page_size=page_size,
            type=type,
            count=count,
            pages=pages,
            last_transaction_id=last_transaction_id,
        )

        list_transactions_response_200.additional_properties = d
        return list_transactions_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="ConfigureAccountBody")


@dataclasses.dataclass
class ConfigureAccountBody:
    """Attributes:
    alias (Optional[str]): Client-defined alias (name) for the Account
    margin_rate (Optional[str]): The string representation of a decimal number."""

    alias: Optional[str]
    margin_rate: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        alias = d.pop("alias", None)
        margin_rate = d.pop("marginRate", None)
        configure_account_body = cls(alias=alias, margin_rate=margin_rate)
        configure_account_body.additional_properties = d
        return configure_account_body

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

ClientPriceStatus = Literal["invalid", "non-tradeable", "tradeable"]

CLIENT_PRICE_STATUS_VALUES: Set[ClientPriceStatus] = {
    "invalid",
    "non-tradeable",
    "tradeable",
}


def check_client_price_status(value: str) -> ClientPriceStatus:
    if value in CLIENT_PRICE_STATUS_VALUES:
        return cast(ClientPriceStatus, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {CLIENT_PRICE_STATUS_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="MarketOrderPositionCloseout")


@dataclasses.dataclass
class MarketOrderPositionCloseout:
    """A MarketOrderPositionCloseout specifies the extensions to a Market Order when it has been created to closeout a
    specific Position.

        Attributes:
            instrument (Optional[str]): The instrument of the Position being closed out.
            units (Optional[str]): Indication of how much of the Position to close. Either "ALL", or a DecimalNumber
                reflection a partial close of the Trade. The DecimalNumber must always be positive, and represent a number that
                doesn't exceed the absolute size of the Position."""

    instrument: Optional[str]
    units: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        market_order_position_closeout = cls(instrument=instrument, units=units)
        market_order_position_closeout.additional_properties = d
        return market_order_position_closeout

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="MarketOrderTradeClose")


@dataclasses.dataclass
class MarketOrderTradeClose:
    """A MarketOrderTradeClose specifies the extensions to a Market Order that has been created specifically to close a
    Trade.

        Attributes:
            trade_id (Optional[str]): The ID of the Trade requested to be closed
            client_trade_id (Optional[str]): The client ID of the Trade requested to be closed
            units (Optional[str]): Indication of how much of the Trade to close. Either "ALL", or a DecimalNumber
                reflection a partial close of the Trade."""

    trade_id: Optional[str]
    client_trade_id: Optional[str]
    units: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        trade_id = d.pop("tradeID", None)
        client_trade_id = d.pop("clientTradeID", None)
        units = d.pop("units", None)
        market_order_trade_close = cls(
            trade_id=trade_id, client_trade_id=client_trade_id, units=units
        )
        market_order_trade_close.additional_properties = d
        return market_order_trade_close

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .fixed_price_order_transaction_position_fill import (
    FixedPriceOrderTransactionPositionFill,
)
from .fixed_price_order_transaction_position_fill import (
    check_fixed_price_order_transaction_position_fill,
)
from .fixed_price_order_transaction_reason import FixedPriceOrderTransactionReason
from .fixed_price_order_transaction_reason import (
    check_fixed_price_order_transaction_reason,
)
from .fixed_price_order_transaction_type import FixedPriceOrderTransactionType
from .fixed_price_order_transaction_type import check_fixed_price_order_transaction_type
from .stop_loss_details import StopLossDetails
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="FixedPriceOrderTransaction")


@dataclasses.dataclass
class FixedPriceOrderTransaction:
    """A FixedPriceOrderTransaction represents the creation of a Fixed Price Order in the user's account. A Fixed Price
    Order is an Order that is filled immediately at a specified price.

        Attributes:
            id (Optional[str]): The Transaction's Identifier.
            time (Optional[str]): The date/time when the Transaction was created.
            user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
            account_id (Optional[str]): The ID of the Account the Transaction was created for.
            batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
                batch are applied to the Account simultaneously.
            request_id (Optional[str]): The Request ID of the request which generated the transaction.
            type (Optional[FixedPriceOrderTransactionType]): The Type of the Transaction. Always set to
                "FIXED_PRICE_ORDER" in a FixedPriceOrderTransaction.
            instrument (Optional[str]): The Fixed Price Order's Instrument.
            units (Optional[str]): The quantity requested to be filled by the Fixed Price Order. A posititive number of
                units results in a long Order, and a negative number of units results in a short Order.
            price (Optional[str]): The price specified for the Fixed Price Order. This price is the exact price that the
                Fixed Price Order will be filled at.
            position_fill (Optional[FixedPriceOrderTransactionPositionFill]): Specification of how Positions in the
                Account are modified when the Order is filled.
            trade_state (Optional[str]): The state that the trade resulting from the Fixed Price Order should be set to.
            reason (Optional[FixedPriceOrderTransactionReason]): The reason that the Fixed Price Order was created
            client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4.
            take_profit_on_fill (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
                Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
                a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
            stop_loss_on_fill (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
                be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
                Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
            trailing_stop_loss_on_fill (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the
                details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
                filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
                is modified directly through the Trade.
            trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
                clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
                your account is associated with MT4."""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[FixedPriceOrderTransactionType]
    instrument: Optional[str]
    units: Optional[str]
    price: Optional[str]
    position_fill: Optional[FixedPriceOrderTransactionPositionFill]
    trade_state: Optional[str]
    reason: Optional[FixedPriceOrderTransactionReason]
    client_extensions: Optional["ClientExtensions"]
    take_profit_on_fill: Optional["TakeProfitDetails"]
    stop_loss_on_fill: Optional["StopLossDetails"]
    trailing_stop_loss_on_fill: Optional["TrailingStopLossDetails"]
    trade_client_extensions: Optional["ClientExtensions"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .take_profit_details import TakeProfitDetails
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .client_extensions import ClientExtensions
        from .stop_loss_details import StopLossDetails

        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[FixedPriceOrderTransactionType]
        if _type is None:
            type = None
        else:
            type = check_fixed_price_order_transaction_type(_type)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        price = d.pop("price", None)
        _position_fill = d.pop("positionFill", None)
        position_fill: Optional[FixedPriceOrderTransactionPositionFill]
        if _position_fill is None:
            position_fill = None
        else:
            position_fill = check_fixed_price_order_transaction_position_fill(
                _position_fill
            )
        trade_state = d.pop("tradeState", None)
        _reason = d.pop("reason", None)
        reason: Optional[FixedPriceOrderTransactionReason]
        if _reason is None:
            reason = None
        else:
            reason = check_fixed_price_order_transaction_reason(_reason)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _take_profit_on_fill = d.pop("takeProfitOnFill", None)
        take_profit_on_fill: Optional[TakeProfitDetails]
        if _take_profit_on_fill is None:
            take_profit_on_fill = None
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)
        _stop_loss_on_fill = d.pop("stopLossOnFill", None)
        stop_loss_on_fill: Optional[StopLossDetails]
        if _stop_loss_on_fill is None:
            stop_loss_on_fill = None
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)
        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", None)
        trailing_stop_loss_on_fill: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss_on_fill is None:
            trailing_stop_loss_on_fill = None
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        fixed_price_order_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            instrument=instrument,
            units=units,
            price=price,
            position_fill=position_fill,
            trade_state=trade_state,
            reason=reason,
            client_extensions=client_extensions,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
        )
        fixed_price_order_transaction.additional_properties = d
        return fixed_price_order_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.client_extensions import ClientExtensions


T = TypeVar("T", bound="SetTradeClientExtensionsBody")


@_attrs_define
class SetTradeClientExtensionsBody:
    """
    Attributes:
        client_extensions (Union[Unset, ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
    """

    client_extensions: Union[Unset, "ClientExtensions"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        client_extensions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.client_extensions, Unset):
            client_extensions = self.client_extensions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if client_extensions is not UNSET:
            field_dict["clientExtensions"] = client_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_extensions import ClientExtensions

        d = src_dict.copy()
        _client_extensions = d.pop("clientExtensions", UNSET)
        client_extensions: Union[Unset, ClientExtensions]
        if isinstance(_client_extensions, Unset):
            client_extensions = UNSET
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)

        set_trade_client_extensions_body = cls(
            client_extensions=client_extensions,
        )

        set_trade_client_extensions_body.additional_properties = d
        return set_trade_client_extensions_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.candlestick_data import CandlestickData


T = TypeVar("T", bound="Candlestick")


@_attrs_define
class Candlestick:
    """The Candlestick representation

    Attributes:
        time (Union[Unset, str]): The start time of the candlestick
        bid (Union[Unset, CandlestickData]): The price data (open, high, low, close) for the Candlestick representation.
        ask (Union[Unset, CandlestickData]): The price data (open, high, low, close) for the Candlestick representation.
        mid (Union[Unset, CandlestickData]): The price data (open, high, low, close) for the Candlestick representation.
        volume (Union[Unset, int]): The number of prices created during the time-range represented by the candlestick.
        complete (Union[Unset, bool]): A flag indicating if the candlestick is complete. A complete candlestick is one
            whose ending time is not in the future.
    """

    time: Union[Unset, str] = UNSET
    bid: Union[Unset, "CandlestickData"] = UNSET
    ask: Union[Unset, "CandlestickData"] = UNSET
    mid: Union[Unset, "CandlestickData"] = UNSET
    volume: Union[Unset, int] = UNSET
    complete: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        time = self.time

        bid: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bid, Unset):
            bid = self.bid.to_dict()

        ask: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.ask, Unset):
            ask = self.ask.to_dict()

        mid: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mid, Unset):
            mid = self.mid.to_dict()

        volume = self.volume

        complete = self.complete

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if time is not UNSET:
            field_dict["time"] = time
        if bid is not UNSET:
            field_dict["bid"] = bid
        if ask is not UNSET:
            field_dict["ask"] = ask
        if mid is not UNSET:
            field_dict["mid"] = mid
        if volume is not UNSET:
            field_dict["volume"] = volume
        if complete is not UNSET:
            field_dict["complete"] = complete

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.candlestick_data import CandlestickData

        d = src_dict.copy()
        time = d.pop("time", UNSET)

        _bid = d.pop("bid", UNSET)
        bid: Union[Unset, CandlestickData]
        if isinstance(_bid, Unset):
            bid = UNSET
        else:
            bid = CandlestickData.from_dict(_bid)

        _ask = d.pop("ask", UNSET)
        ask: Union[Unset, CandlestickData]
        if isinstance(_ask, Unset):
            ask = UNSET
        else:
            ask = CandlestickData.from_dict(_ask)

        _mid = d.pop("mid", UNSET)
        mid: Union[Unset, CandlestickData]
        if isinstance(_mid, Unset):
            mid = UNSET
        else:
            mid = CandlestickData.from_dict(_mid)

        volume = d.pop("volume", UNSET)

        complete = d.pop("complete", UNSET)

        candlestick = cls(
            time=time,
            bid=bid,
            ask=ask,
            mid=mid,
            volume=volume,
            complete=complete,
        )

        candlestick.additional_properties = d
        return candlestick

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_price import ClientPrice
from .home_conversions import HomeConversions
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="GetPricesResponse200")


@dataclasses.dataclass
class GetPricesResponse200:
    """Attributes:
    prices (Optional[List['ClientPrice']]): The list of Price objects requested.
    home_conversions (Optional[List['HomeConversions']]): The list of home currency conversion factors
        requested. This field will only be present if includeHomeConversions was set to true in the request.
    time (Optional[str]): The DateTime value to use for the "since" parameter in the next poll request."""

    prices: Optional[List["ClientPrice"]]
    home_conversions: Optional[List["HomeConversions"]]
    time: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .home_conversions import HomeConversions
        from .client_price import ClientPrice

        d = src_dict.copy()
        prices = []
        _prices = d.pop("prices", None)
        for prices_item_data in _prices or []:
            prices_item = ClientPrice.from_dict(prices_item_data)
            prices.append(prices_item)
        home_conversions = []
        _home_conversions = d.pop("homeConversions", None)
        for home_conversions_item_data in _home_conversions or []:
            home_conversions_item = HomeConversions.from_dict(
                home_conversions_item_data
            )
            home_conversions.append(home_conversions_item)
        time = d.pop("time", None)
        get_prices_response_200 = cls(
            prices=prices, home_conversions=home_conversions, time=time
        )
        get_prices_response_200.additional_properties = d
        return get_prices_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Literal, Set, cast

FixedPriceOrderReason = Literal["PLATFORM_ACCOUNT_MIGRATION"]
FIXED_PRICE_ORDER_REASON_VALUES: Set[FixedPriceOrderReason] = {
    "PLATFORM_ACCOUNT_MIGRATION"
}


def check_fixed_price_order_reason(value: str) -> FixedPriceOrderReason:
    if value in FIXED_PRICE_ORDER_REASON_VALUES:
        return cast(FixedPriceOrderReason, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {FIXED_PRICE_ORDER_REASON_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .trade import Trade
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="GetTradeResponse200")


@dataclasses.dataclass
class GetTradeResponse200:
    """Attributes:
    trade (Optional[Trade]): The specification of a Trade within an Account. This includes the full
        representation of the Trade's dependent Orders in addition to the IDs of those Orders.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    trade: Optional["Trade"]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .trade import Trade

        d = src_dict.copy()
        _trade = d.pop("trade", None)
        trade: Optional[Trade]
        if _trade is None:
            trade = None
        else:
            trade = Trade.from_dict(_trade)
        last_transaction_id = d.pop("lastTransactionID", None)
        get_trade_response_200 = cls(
            trade=trade, last_transaction_id=last_transaction_id
        )
        get_trade_response_200.additional_properties = d
        return get_trade_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

LimitOrderPositionFill = Literal["DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"]

LIMIT_ORDER_POSITION_FILL_VALUES: Set[LimitOrderPositionFill] = {
    "DEFAULT",
    "OPEN_ONLY",
    "REDUCE_FIRST",
    "REDUCE_ONLY",
}


def check_limit_order_position_fill(value: str) -> LimitOrderPositionFill:
    if value in LIMIT_ORDER_POSITION_FILL_VALUES:
        return cast(LimitOrderPositionFill, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {LIMIT_ORDER_POSITION_FILL_VALUES!r}"
    )



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="CalculatedTradeState")


@_attrs_define
class CalculatedTradeState:
    """The dynamic (calculated) state of an open Trade

    Attributes:
        id (Union[Unset, str]): The Trade's ID.
        unrealized_pl (Union[Unset, str]): The Trade's unrealized profit/loss.
        margin_used (Union[Unset, str]): Margin currently used by the Trade.
    """

    id: Union[Unset, str] = UNSET
    unrealized_pl: Union[Unset, str] = UNSET
    margin_used: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        unrealized_pl = self.unrealized_pl

        margin_used = self.margin_used

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if unrealized_pl is not UNSET:
            field_dict["unrealizedPL"] = unrealized_pl
        if margin_used is not UNSET:
            field_dict["marginUsed"] = margin_used

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        unrealized_pl = d.pop("unrealizedPL", UNSET)

        margin_used = d.pop("marginUsed", UNSET)

        calculated_trade_state = cls(
            id=id,
            unrealized_pl=unrealized_pl,
            margin_used=margin_used,
        )

        calculated_trade_state.additional_properties = d
        return calculated_trade_state

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.open_trade_financing import OpenTradeFinancing


T = TypeVar("T", bound="PositionFinancing")


@_attrs_define
class PositionFinancing:
    """OpenTradeFinancing is used to pay/collect daily financing charge for a Position within an Account

    Attributes:
        instrument (Union[Unset, str]): The instrument of the Position that financing is being paid/collected for.
        financing (Union[Unset, str]): The amount of financing paid/collected for the Position.
        open_trade_financings (Union[Unset, List['OpenTradeFinancing']]): The financing paid/collecte for each open
            Trade within the Position.
    """

    instrument: Union[Unset, str] = UNSET
    financing: Union[Unset, str] = UNSET
    open_trade_financings: Union[Unset, List["OpenTradeFinancing"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        instrument = self.instrument

        financing = self.financing

        open_trade_financings: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.open_trade_financings, Unset):
            open_trade_financings = []
            for open_trade_financings_item_data in self.open_trade_financings:
                open_trade_financings_item = open_trade_financings_item_data.to_dict()
                open_trade_financings.append(open_trade_financings_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if instrument is not UNSET:
            field_dict["instrument"] = instrument
        if financing is not UNSET:
            field_dict["financing"] = financing
        if open_trade_financings is not UNSET:
            field_dict["openTradeFinancings"] = open_trade_financings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.open_trade_financing import OpenTradeFinancing

        d = src_dict.copy()
        instrument = d.pop("instrument", UNSET)

        financing = d.pop("financing", UNSET)

        open_trade_financings = []
        _open_trade_financings = d.pop("openTradeFinancings", UNSET)
        for open_trade_financings_item_data in _open_trade_financings or []:
            open_trade_financings_item = OpenTradeFinancing.from_dict(
                open_trade_financings_item_data
            )

            open_trade_financings.append(open_trade_financings_item)

        position_financing = cls(
            instrument=instrument,
            financing=financing,
            open_trade_financings=open_trade_financings,
        )

        position_financing.additional_properties = d
        return position_financing

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .accept_datetime_format import AcceptDatetimeFormat
from .account import Account
from .account_changes import AccountChanges
from .account_changes_state import AccountChangesState
from .account_financing_mode import AccountFinancingMode
from .account_guaranteed_stop_loss_order_mode import AccountGuaranteedStopLossOrderMode
from .account_properties import AccountProperties
from .account_summary import AccountSummary
from .account_summary_guaranteed_stop_loss_order_mode import (
    AccountSummaryGuaranteedStopLossOrderMode,
)
from .calculated_account_state import CalculatedAccountState
from .calculated_position_state import CalculatedPositionState
from .calculated_trade_state import CalculatedTradeState
from .cancel_order_response_200 import CancelOrderResponse200
from .cancel_order_response_404 import CancelOrderResponse404
from .cancellable_order_type import CancellableOrderType
from .candlestick import Candlestick
from .candlestick_data import CandlestickData
from .candlestick_granularity import CandlestickGranularity
from .client_configure_reject_transaction import ClientConfigureRejectTransaction
from .client_configure_reject_transaction_reject_reason import (
    ClientConfigureRejectTransactionRejectReason,
)
from .client_configure_reject_transaction_type import (
    ClientConfigureRejectTransactionType,
)
from .client_configure_transaction import ClientConfigureTransaction
from .client_configure_transaction_type import ClientConfigureTransactionType
from .client_extensions import ClientExtensions
from .client_price import ClientPrice
from .client_price_status import ClientPriceStatus
from .close_position_body import ClosePositionBody
from .close_position_response_200 import ClosePositionResponse200
from .close_position_response_400 import ClosePositionResponse400
from .close_position_response_404 import ClosePositionResponse404
from .close_trade_body import CloseTradeBody
from .close_trade_response_200 import CloseTradeResponse200
from .close_trade_response_400 import CloseTradeResponse400
from .close_trade_response_404 import CloseTradeResponse404
from .close_transaction import CloseTransaction
from .close_transaction_type import CloseTransactionType
from .configure_account_body import ConfigureAccountBody
from .configure_account_response_200 import ConfigureAccountResponse200
from .configure_account_response_400 import ConfigureAccountResponse400
from .configure_account_response_403 import ConfigureAccountResponse403
from .create_order_body import CreateOrderBody
from .create_order_response_201 import CreateOrderResponse201
from .create_order_response_400 import CreateOrderResponse400
from .create_order_response_404 import CreateOrderResponse404
from .create_transaction import CreateTransaction
from .create_transaction_type import CreateTransactionType
from .daily_financing_transaction import DailyFinancingTransaction
from .daily_financing_transaction_account_financing_mode import (
    DailyFinancingTransactionAccountFinancingMode,
)
from .daily_financing_transaction_type import DailyFinancingTransactionType
from .delayed_trade_closure_transaction import DelayedTradeClosureTransaction
from .delayed_trade_closure_transaction_reason import (
    DelayedTradeClosureTransactionReason,
)
from .delayed_trade_closure_transaction_type import DelayedTradeClosureTransactionType
from .direction import Direction
from .dynamic_order_state import DynamicOrderState
from .fixed_price_order import FixedPriceOrder
from .fixed_price_order_position_fill import FixedPriceOrderPositionFill
from .fixed_price_order_reason import FixedPriceOrderReason
from .fixed_price_order_state import FixedPriceOrderState
from .fixed_price_order_transaction import FixedPriceOrderTransaction
from .fixed_price_order_transaction_position_fill import (
    FixedPriceOrderTransactionPositionFill,
)
from .fixed_price_order_transaction_reason import FixedPriceOrderTransactionReason
from .fixed_price_order_transaction_type import FixedPriceOrderTransactionType
from .fixed_price_order_type import FixedPriceOrderType
from .funding_reason import FundingReason
from .get_account_changes_response_200 import GetAccountChangesResponse200
from .get_account_instruments_response_200 import GetAccountInstrumentsResponse200
from .get_account_response_200 import GetAccountResponse200
from .get_account_summary_response_200 import GetAccountSummaryResponse200
from .get_base_prices_response_200 import GetBasePricesResponse200
from .get_external_user_info_response_200 import GetExternalUserInfoResponse200
from .get_instrument_candles_response_200 import GetInstrumentCandlesResponse200
from .get_instrument_candles_response_200_granularity import (
    GetInstrumentCandlesResponse200Granularity,
)
from .get_instrument_price_range_response_200 import GetInstrumentPriceRangeResponse200
from .get_instrument_price_response_200 import GetInstrumentPriceResponse200
from .get_instruments_instrument_order_book_response_200 import (
    GetInstrumentsInstrumentOrderBookResponse200,
)
from .get_instruments_instrument_position_book_response_200 import (
    GetInstrumentsInstrumentPositionBookResponse200,
)
from .get_order_response_200 import GetOrderResponse200
from .get_position_response_200 import GetPositionResponse200
from .get_price_range_response_200 import GetPriceRangeResponse200
from .get_prices_response_200 import GetPricesResponse200
from .get_trade_response_200 import GetTradeResponse200
from .get_transaction_range_response_200 import GetTransactionRangeResponse200
from .get_transaction_response_200 import GetTransactionResponse200
from .get_transactions_since_id_response_200 import GetTransactionsSinceIdResponse200
from .get_user_info_response_200 import GetUserInfoResponse200
from .guaranteed_stop_loss_order_entry_data import GuaranteedStopLossOrderEntryData
from .guaranteed_stop_loss_order_level_restriction import (
    GuaranteedStopLossOrderLevelRestriction,
)
from .guaranteed_stop_loss_order_mode import GuaranteedStopLossOrderMode
from .home_conversions import HomeConversions
from .instrument import Instrument
from .instrument_commission import InstrumentCommission
from .instrument_type import InstrumentType
from .limit_order import LimitOrder
from .limit_order_position_fill import LimitOrderPositionFill
from .limit_order_reason import LimitOrderReason
from .limit_order_reject_transaction import LimitOrderRejectTransaction
from .limit_order_reject_transaction_position_fill import (
    LimitOrderRejectTransactionPositionFill,
)
from .limit_order_reject_transaction_reason import LimitOrderRejectTransactionReason
from .limit_order_reject_transaction_reject_reason import (
    LimitOrderRejectTransactionRejectReason,
)
from .limit_order_reject_transaction_time_in_force import (
    LimitOrderRejectTransactionTimeInForce,
)
from .limit_order_reject_transaction_trigger_condition import (
    LimitOrderRejectTransactionTriggerCondition,
)
from .limit_order_reject_transaction_type import LimitOrderRejectTransactionType
from .limit_order_request import LimitOrderRequest
from .limit_order_request_position_fill import LimitOrderRequestPositionFill
from .limit_order_request_time_in_force import LimitOrderRequestTimeInForce
from .limit_order_request_trigger_condition import LimitOrderRequestTriggerCondition
from .limit_order_request_type import LimitOrderRequestType
from .limit_order_state import LimitOrderState
from .limit_order_time_in_force import LimitOrderTimeInForce
from .limit_order_transaction import LimitOrderTransaction
from .limit_order_transaction_position_fill import LimitOrderTransactionPositionFill
from .limit_order_transaction_reason import LimitOrderTransactionReason
from .limit_order_transaction_time_in_force import LimitOrderTransactionTimeInForce
from .limit_order_transaction_trigger_condition import (
    LimitOrderTransactionTriggerCondition,
)
from .limit_order_transaction_type import LimitOrderTransactionType
from .limit_order_trigger_condition import LimitOrderTriggerCondition
from .limit_order_type import LimitOrderType
from .liquidity_regeneration_schedule import LiquidityRegenerationSchedule
from .liquidity_regeneration_schedule_step import LiquidityRegenerationScheduleStep
from .list_accounts_response_200 import ListAccountsResponse200
from .list_open_positions_response_200 import ListOpenPositionsResponse200
from .list_open_trades_response_200 import ListOpenTradesResponse200
from .list_orders_response_200 import ListOrdersResponse200
from .list_pending_orders_response_200 import ListPendingOrdersResponse200
from .list_positions_response_200 import ListPositionsResponse200
from .list_trades_response_200 import ListTradesResponse200
from .list_transactions_response_200 import ListTransactionsResponse200
from .list_transactions_response_200_type_item import (
    ListTransactionsResponse200TypeItem,
)
from .margin_call_enter_transaction import MarginCallEnterTransaction
from .margin_call_enter_transaction_type import MarginCallEnterTransactionType
from .margin_call_exit_transaction import MarginCallExitTransaction
from .margin_call_exit_transaction_type import MarginCallExitTransactionType
from .margin_call_extend_transaction import MarginCallExtendTransaction
from .margin_call_extend_transaction_type import MarginCallExtendTransactionType
from .market_if_touched_order import MarketIfTouchedOrder
from .market_if_touched_order_position_fill import MarketIfTouchedOrderPositionFill
from .market_if_touched_order_reason import MarketIfTouchedOrderReason
from .market_if_touched_order_reject_transaction import (
    MarketIfTouchedOrderRejectTransaction,
)
from .market_if_touched_order_reject_transaction_position_fill import (
    MarketIfTouchedOrderRejectTransactionPositionFill,
)
from .market_if_touched_order_reject_transaction_reason import (
    MarketIfTouchedOrderRejectTransactionReason,
)
from .market_if_touched_order_reject_transaction_reject_reason import (
    MarketIfTouchedOrderRejectTransactionRejectReason,
)
from .market_if_touched_order_reject_transaction_time_in_force import (
    MarketIfTouchedOrderRejectTransactionTimeInForce,
)
from .market_if_touched_order_reject_transaction_trigger_condition import (
    MarketIfTouchedOrderRejectTransactionTriggerCondition,
)
from .market_if_touched_order_reject_transaction_type import (
    MarketIfTouchedOrderRejectTransactionType,
)
from .market_if_touched_order_request import MarketIfTouchedOrderRequest
from .market_if_touched_order_request_position_fill import (
    MarketIfTouchedOrderRequestPositionFill,
)
from .market_if_touched_order_request_time_in_force import (
    MarketIfTouchedOrderRequestTimeInForce,
)
from .market_if_touched_order_request_trigger_condition import (
    MarketIfTouchedOrderRequestTriggerCondition,
)
from .market_if_touched_order_request_type import MarketIfTouchedOrderRequestType
from .market_if_touched_order_state import MarketIfTouchedOrderState
from .market_if_touched_order_time_in_force import MarketIfTouchedOrderTimeInForce
from .market_if_touched_order_transaction import MarketIfTouchedOrderTransaction
from .market_if_touched_order_transaction_position_fill import (
    MarketIfTouchedOrderTransactionPositionFill,
)
from .market_if_touched_order_transaction_reason import (
    MarketIfTouchedOrderTransactionReason,
)
from .market_if_touched_order_transaction_time_in_force import (
    MarketIfTouchedOrderTransactionTimeInForce,
)
from .market_if_touched_order_transaction_trigger_condition import (
    MarketIfTouchedOrderTransactionTriggerCondition,
)
from .market_if_touched_order_transaction_type import (
    MarketIfTouchedOrderTransactionType,
)
from .market_if_touched_order_trigger_condition import (
    MarketIfTouchedOrderTriggerCondition,
)
from .market_if_touched_order_type import MarketIfTouchedOrderType
from .market_order import MarketOrder
from .market_order_delayed_trade_close import MarketOrderDelayedTradeClose
from .market_order_margin_closeout import MarketOrderMarginCloseout
from .market_order_margin_closeout_reason import MarketOrderMarginCloseoutReason
from .market_order_position_closeout import MarketOrderPositionCloseout
from .market_order_position_fill import MarketOrderPositionFill
from .market_order_reason import MarketOrderReason
from .market_order_reject_transaction import MarketOrderRejectTransaction
from .market_order_reject_transaction_position_fill import (
    MarketOrderRejectTransactionPositionFill,
)
from .market_order_reject_transaction_reason import MarketOrderRejectTransactionReason
from .market_order_reject_transaction_reject_reason import (
    MarketOrderRejectTransactionRejectReason,
)
from .market_order_reject_transaction_time_in_force import (
    MarketOrderRejectTransactionTimeInForce,
)
from .market_order_reject_transaction_type import MarketOrderRejectTransactionType
from .market_order_request import MarketOrderRequest
from .market_order_request_position_fill import MarketOrderRequestPositionFill
from .market_order_request_time_in_force import MarketOrderRequestTimeInForce
from .market_order_request_type import MarketOrderRequestType
from .market_order_state import MarketOrderState
from .market_order_time_in_force import MarketOrderTimeInForce
from .market_order_trade_close import MarketOrderTradeClose
from .market_order_transaction import MarketOrderTransaction
from .market_order_transaction_position_fill import MarketOrderTransactionPositionFill
from .market_order_transaction_reason import MarketOrderTransactionReason
from .market_order_transaction_time_in_force import MarketOrderTransactionTimeInForce
from .market_order_transaction_type import MarketOrderTransactionType
from .market_order_type import MarketOrderType
from .mt4_transaction_heartbeat import MT4TransactionHeartbeat
from .open_trade_financing import OpenTradeFinancing
from .order import Order
from .order_book import OrderBook
from .order_book_bucket import OrderBookBucket
from .order_cancel_reason import OrderCancelReason
from .order_cancel_reject_transaction import OrderCancelRejectTransaction
from .order_cancel_reject_transaction_reject_reason import (
    OrderCancelRejectTransactionRejectReason,
)
from .order_cancel_reject_transaction_type import OrderCancelRejectTransactionType
from .order_cancel_transaction import OrderCancelTransaction
from .order_cancel_transaction_reason import OrderCancelTransactionReason
from .order_cancel_transaction_type import OrderCancelTransactionType
from .order_client_extensions_modify_reject_transaction import (
    OrderClientExtensionsModifyRejectTransaction,
)
from .order_client_extensions_modify_reject_transaction_reject_reason import (
    OrderClientExtensionsModifyRejectTransactionRejectReason,
)
from .order_client_extensions_modify_reject_transaction_type import (
    OrderClientExtensionsModifyRejectTransactionType,
)
from .order_client_extensions_modify_transaction import (
    OrderClientExtensionsModifyTransaction,
)
from .order_client_extensions_modify_transaction_type import (
    OrderClientExtensionsModifyTransactionType,
)
from .order_fill_reason import OrderFillReason
from .order_fill_transaction import OrderFillTransaction
from .order_fill_transaction_reason import OrderFillTransactionReason
from .order_fill_transaction_type import OrderFillTransactionType
from .order_identifier import OrderIdentifier
from .order_position_fill import OrderPositionFill
from .order_request import OrderRequest
from .order_state import OrderState
from .order_state_filter import OrderStateFilter
from .order_trigger_condition import OrderTriggerCondition
from .order_type import OrderType
from .position import Position
from .position_aggregation_mode import PositionAggregationMode
from .position_book import PositionBook
from .position_book_bucket import PositionBookBucket
from .position_financing import PositionFinancing
from .position_side import PositionSide
from .price import Price
from .price_bucket import PriceBucket
from .price_status import PriceStatus
from .pricing_heartbeat import PricingHeartbeat
from .quote_home_conversion_factors import QuoteHomeConversionFactors
from .reopen_transaction import ReopenTransaction
from .reopen_transaction_type import ReopenTransactionType
from .replace_order_body import ReplaceOrderBody
from .replace_order_response_201 import ReplaceOrderResponse201
from .replace_order_response_400 import ReplaceOrderResponse400
from .replace_order_response_404 import ReplaceOrderResponse404
from .reset_resettable_pl_transaction import ResetResettablePLTransaction
from .reset_resettable_pl_transaction_type import ResetResettablePLTransactionType
from .set_order_client_extensions_body import SetOrderClientExtensionsBody
from .set_order_client_extensions_response_200 import (
    SetOrderClientExtensionsResponse200,
)
from .set_order_client_extensions_response_400 import (
    SetOrderClientExtensionsResponse400,
)
from .set_order_client_extensions_response_404 import (
    SetOrderClientExtensionsResponse404,
)
from .set_trade_client_extensions_body import SetTradeClientExtensionsBody
from .set_trade_client_extensions_response_200 import (
    SetTradeClientExtensionsResponse200,
)
from .set_trade_client_extensions_response_400 import (
    SetTradeClientExtensionsResponse400,
)
from .set_trade_client_extensions_response_404 import (
    SetTradeClientExtensionsResponse404,
)
from .set_trade_dependent_orders_body import SetTradeDependentOrdersBody
from .set_trade_dependent_orders_response_200 import SetTradeDependentOrdersResponse200
from .set_trade_dependent_orders_response_400 import SetTradeDependentOrdersResponse400
from .stop_loss_details import StopLossDetails
from .stop_loss_details_time_in_force import StopLossDetailsTimeInForce
from .stop_loss_order import StopLossOrder
from .stop_loss_order_reason import StopLossOrderReason
from .stop_loss_order_reject_transaction import StopLossOrderRejectTransaction
from .stop_loss_order_reject_transaction_reason import (
    StopLossOrderRejectTransactionReason,
)
from .stop_loss_order_reject_transaction_reject_reason import (
    StopLossOrderRejectTransactionRejectReason,
)
from .stop_loss_order_reject_transaction_time_in_force import (
    StopLossOrderRejectTransactionTimeInForce,
)
from .stop_loss_order_reject_transaction_trigger_condition import (
    StopLossOrderRejectTransactionTriggerCondition,
)
from .stop_loss_order_reject_transaction_type import StopLossOrderRejectTransactionType
from .stop_loss_order_request import StopLossOrderRequest
from .stop_loss_order_request_time_in_force import StopLossOrderRequestTimeInForce
from .stop_loss_order_request_trigger_condition import (
    StopLossOrderRequestTriggerCondition,
)
from .stop_loss_order_request_type import StopLossOrderRequestType
from .stop_loss_order_state import StopLossOrderState
from .stop_loss_order_time_in_force import StopLossOrderTimeInForce
from .stop_loss_order_transaction import StopLossOrderTransaction
from .stop_loss_order_transaction_reason import StopLossOrderTransactionReason
from .stop_loss_order_transaction_time_in_force import (
    StopLossOrderTransactionTimeInForce,
)
from .stop_loss_order_transaction_trigger_condition import (
    StopLossOrderTransactionTriggerCondition,
)
from .stop_loss_order_transaction_type import StopLossOrderTransactionType
from .stop_loss_order_trigger_condition import StopLossOrderTriggerCondition
from .stop_loss_order_type import StopLossOrderType
from .stop_order import StopOrder
from .stop_order_position_fill import StopOrderPositionFill
from .stop_order_reason import StopOrderReason
from .stop_order_reject_transaction import StopOrderRejectTransaction
from .stop_order_reject_transaction_position_fill import (
    StopOrderRejectTransactionPositionFill,
)
from .stop_order_reject_transaction_reason import StopOrderRejectTransactionReason
from .stop_order_reject_transaction_reject_reason import (
    StopOrderRejectTransactionRejectReason,
)
from .stop_order_reject_transaction_time_in_force import (
    StopOrderRejectTransactionTimeInForce,
)
from .stop_order_reject_transaction_trigger_condition import (
    StopOrderRejectTransactionTriggerCondition,
)
from .stop_order_reject_transaction_type import StopOrderRejectTransactionType
from .stop_order_request import StopOrderRequest
from .stop_order_request_position_fill import StopOrderRequestPositionFill
from .stop_order_request_time_in_force import StopOrderRequestTimeInForce
from .stop_order_request_trigger_condition import StopOrderRequestTriggerCondition
from .stop_order_request_type import StopOrderRequestType
from .stop_order_state import StopOrderState
from .stop_order_time_in_force import StopOrderTimeInForce
from .stop_order_transaction import StopOrderTransaction
from .stop_order_transaction_position_fill import StopOrderTransactionPositionFill
from .stop_order_transaction_reason import StopOrderTransactionReason
from .stop_order_transaction_time_in_force import StopOrderTransactionTimeInForce
from .stop_order_transaction_trigger_condition import (
    StopOrderTransactionTriggerCondition,
)
from .stop_order_transaction_type import StopOrderTransactionType
from .stop_order_trigger_condition import StopOrderTriggerCondition
from .stop_order_type import StopOrderType
from .stream_pricing_response_200 import StreamPricingResponse200
from .stream_transactions_response_200 import StreamTransactionsResponse200
from .take_profit_details import TakeProfitDetails
from .take_profit_details_time_in_force import TakeProfitDetailsTimeInForce
from .take_profit_order import TakeProfitOrder
from .take_profit_order_reason import TakeProfitOrderReason
from .take_profit_order_reject_transaction import TakeProfitOrderRejectTransaction
from .take_profit_order_reject_transaction_reason import (
    TakeProfitOrderRejectTransactionReason,
)
from .take_profit_order_reject_transaction_reject_reason import (
    TakeProfitOrderRejectTransactionRejectReason,
)
from .take_profit_order_reject_transaction_time_in_force import (
    TakeProfitOrderRejectTransactionTimeInForce,
)
from .take_profit_order_reject_transaction_trigger_condition import (
    TakeProfitOrderRejectTransactionTriggerCondition,
)
from .take_profit_order_reject_transaction_type import (
    TakeProfitOrderRejectTransactionType,
)
from .take_profit_order_request import TakeProfitOrderRequest
from .take_profit_order_request_time_in_force import TakeProfitOrderRequestTimeInForce
from .take_profit_order_request_trigger_condition import (
    TakeProfitOrderRequestTriggerCondition,
)
from .take_profit_order_request_type import TakeProfitOrderRequestType
from .take_profit_order_state import TakeProfitOrderState
from .take_profit_order_time_in_force import TakeProfitOrderTimeInForce
from .take_profit_order_transaction import TakeProfitOrderTransaction
from .take_profit_order_transaction_reason import TakeProfitOrderTransactionReason
from .take_profit_order_transaction_time_in_force import (
    TakeProfitOrderTransactionTimeInForce,
)
from .take_profit_order_transaction_trigger_condition import (
    TakeProfitOrderTransactionTriggerCondition,
)
from .take_profit_order_transaction_type import TakeProfitOrderTransactionType
from .take_profit_order_trigger_condition import TakeProfitOrderTriggerCondition
from .take_profit_order_type import TakeProfitOrderType
from .time_in_force import TimeInForce
from .trade import Trade
from .trade_client_extensions_modify_reject_transaction import (
    TradeClientExtensionsModifyRejectTransaction,
)
from .trade_client_extensions_modify_reject_transaction_reject_reason import (
    TradeClientExtensionsModifyRejectTransactionRejectReason,
)
from .trade_client_extensions_modify_reject_transaction_type import (
    TradeClientExtensionsModifyRejectTransactionType,
)
from .trade_client_extensions_modify_transaction import (
    TradeClientExtensionsModifyTransaction,
)
from .trade_client_extensions_modify_transaction_type import (
    TradeClientExtensionsModifyTransactionType,
)
from .trade_open import TradeOpen
from .trade_pl import TradePL
from .trade_reduce import TradeReduce
from .trade_state import TradeState
from .trade_state_filter import TradeStateFilter
from .trade_summary import TradeSummary
from .trade_summary_state import TradeSummaryState
from .trailing_stop_loss_details import TrailingStopLossDetails
from .trailing_stop_loss_details_time_in_force import TrailingStopLossDetailsTimeInForce
from .trailing_stop_loss_order import TrailingStopLossOrder
from .trailing_stop_loss_order_reason import TrailingStopLossOrderReason
from .trailing_stop_loss_order_reject_transaction import (
    TrailingStopLossOrderRejectTransaction,
)
from .trailing_stop_loss_order_reject_transaction_reason import (
    TrailingStopLossOrderRejectTransactionReason,
)
from .trailing_stop_loss_order_reject_transaction_reject_reason import (
    TrailingStopLossOrderRejectTransactionRejectReason,
)
from .trailing_stop_loss_order_reject_transaction_time_in_force import (
    TrailingStopLossOrderRejectTransactionTimeInForce,
)
from .trailing_stop_loss_order_reject_transaction_trigger_condition import (
    TrailingStopLossOrderRejectTransactionTriggerCondition,
)
from .trailing_stop_loss_order_reject_transaction_type import (
    TrailingStopLossOrderRejectTransactionType,
)
from .trailing_stop_loss_order_request import TrailingStopLossOrderRequest
from .trailing_stop_loss_order_request_time_in_force import (
    TrailingStopLossOrderRequestTimeInForce,
)
from .trailing_stop_loss_order_request_trigger_condition import (
    TrailingStopLossOrderRequestTriggerCondition,
)
from .trailing_stop_loss_order_request_type import TrailingStopLossOrderRequestType
from .trailing_stop_loss_order_state import TrailingStopLossOrderState
from .trailing_stop_loss_order_time_in_force import TrailingStopLossOrderTimeInForce
from .trailing_stop_loss_order_transaction import TrailingStopLossOrderTransaction
from .trailing_stop_loss_order_transaction_reason import (
    TrailingStopLossOrderTransactionReason,
)
from .trailing_stop_loss_order_transaction_time_in_force import (
    TrailingStopLossOrderTransactionTimeInForce,
)
from .trailing_stop_loss_order_transaction_trigger_condition import (
    TrailingStopLossOrderTransactionTriggerCondition,
)
from .trailing_stop_loss_order_transaction_type import (
    TrailingStopLossOrderTransactionType,
)
from .trailing_stop_loss_order_trigger_condition import (
    TrailingStopLossOrderTriggerCondition,
)
from .trailing_stop_loss_order_type import TrailingStopLossOrderType
from .transaction import Transaction
from .transaction_filter import TransactionFilter
from .transaction_heartbeat import TransactionHeartbeat
from .transaction_reject_reason import TransactionRejectReason
from .transaction_type import TransactionType
from .transfer_funds_reject_transaction import TransferFundsRejectTransaction
from .transfer_funds_reject_transaction_funding_reason import (
    TransferFundsRejectTransactionFundingReason,
)
from .transfer_funds_reject_transaction_reject_reason import (
    TransferFundsRejectTransactionRejectReason,
)
from .transfer_funds_reject_transaction_type import TransferFundsRejectTransactionType
from .transfer_funds_transaction import TransferFundsTransaction
from .transfer_funds_transaction_funding_reason import (
    TransferFundsTransactionFundingReason,
)
from .transfer_funds_transaction_type import TransferFundsTransactionType
from .units_available import UnitsAvailable
from .units_available_details import UnitsAvailableDetails
from .user_info import UserInfo
from .user_info_external import UserInfoExternal
from .weekly_alignment import WeeklyAlignment

__all__ = (
    "AcceptDatetimeFormat",
    "Account",
    "AccountChanges",
    "AccountChangesState",
    "AccountFinancingMode",
    "AccountGuaranteedStopLossOrderMode",
    "AccountProperties",
    "AccountSummary",
    "AccountSummaryGuaranteedStopLossOrderMode",
    "CalculatedAccountState",
    "CalculatedPositionState",
    "CalculatedTradeState",
    "CancellableOrderType",
    "CancelOrderResponse200",
    "CancelOrderResponse404",
    "Candlestick",
    "CandlestickData",
    "CandlestickGranularity",
    "ClientConfigureRejectTransaction",
    "ClientConfigureRejectTransactionRejectReason",
    "ClientConfigureRejectTransactionType",
    "ClientConfigureTransaction",
    "ClientConfigureTransactionType",
    "ClientExtensions",
    "ClientPrice",
    "ClientPriceStatus",
    "ClosePositionBody",
    "ClosePositionResponse200",
    "ClosePositionResponse400",
    "ClosePositionResponse404",
    "CloseTradeBody",
    "CloseTradeResponse200",
    "CloseTradeResponse400",
    "CloseTradeResponse404",
    "CloseTransaction",
    "CloseTransactionType",
    "ConfigureAccountBody",
    "ConfigureAccountResponse200",
    "ConfigureAccountResponse400",
    "ConfigureAccountResponse403",
    "CreateOrderBody",
    "CreateOrderResponse201",
    "CreateOrderResponse400",
    "CreateOrderResponse404",
    "CreateTransaction",
    "CreateTransactionType",
    "DailyFinancingTransaction",
    "DailyFinancingTransactionAccountFinancingMode",
    "DailyFinancingTransactionType",
    "DelayedTradeClosureTransaction",
    "DelayedTradeClosureTransactionReason",
    "DelayedTradeClosureTransactionType",
    "Direction",
    "DynamicOrderState",
    "FixedPriceOrder",
    "FixedPriceOrderPositionFill",
    "FixedPriceOrderReason",
    "FixedPriceOrderState",
    "FixedPriceOrderTransaction",
    "FixedPriceOrderTransactionPositionFill",
    "FixedPriceOrderTransactionReason",
    "FixedPriceOrderTransactionType",
    "FixedPriceOrderType",
    "FundingReason",
    "GetAccountChangesResponse200",
    "GetAccountInstrumentsResponse200",
    "GetAccountResponse200",
    "GetAccountSummaryResponse200",
    "GetBasePricesResponse200",
    "GetExternalUserInfoResponse200",
    "GetInstrumentCandlesResponse200",
    "GetInstrumentCandlesResponse200Granularity",
    "GetInstrumentPriceRangeResponse200",
    "GetInstrumentPriceResponse200",
    "GetInstrumentsInstrumentOrderBookResponse200",
    "GetInstrumentsInstrumentPositionBookResponse200",
    "GetOrderResponse200",
    "GetPositionResponse200",
    "GetPriceRangeResponse200",
    "GetPricesResponse200",
    "GetTradeResponse200",
    "GetTransactionRangeResponse200",
    "GetTransactionResponse200",
    "GetTransactionsSinceIdResponse200",
    "GetUserInfoResponse200",
    "GuaranteedStopLossOrderEntryData",
    "GuaranteedStopLossOrderLevelRestriction",
    "GuaranteedStopLossOrderMode",
    "HomeConversions",
    "Instrument",
    "InstrumentCommission",
    "InstrumentType",
    "LimitOrder",
    "LimitOrderPositionFill",
    "LimitOrderReason",
    "LimitOrderRejectTransaction",
    "LimitOrderRejectTransactionPositionFill",
    "LimitOrderRejectTransactionReason",
    "LimitOrderRejectTransactionRejectReason",
    "LimitOrderRejectTransactionTimeInForce",
    "LimitOrderRejectTransactionTriggerCondition",
    "LimitOrderRejectTransactionType",
    "LimitOrderRequest",
    "LimitOrderRequestPositionFill",
    "LimitOrderRequestTimeInForce",
    "LimitOrderRequestTriggerCondition",
    "LimitOrderRequestType",
    "LimitOrderState",
    "LimitOrderTimeInForce",
    "LimitOrderTransaction",
    "LimitOrderTransactionPositionFill",
    "LimitOrderTransactionReason",
    "LimitOrderTransactionTimeInForce",
    "LimitOrderTransactionTriggerCondition",
    "LimitOrderTransactionType",
    "LimitOrderTriggerCondition",
    "LimitOrderType",
    "LiquidityRegenerationSchedule",
    "LiquidityRegenerationScheduleStep",
    "ListAccountsResponse200",
    "ListOpenPositionsResponse200",
    "ListOpenTradesResponse200",
    "ListOrdersResponse200",
    "ListPendingOrdersResponse200",
    "ListPositionsResponse200",
    "ListTradesResponse200",
    "ListTransactionsResponse200",
    "ListTransactionsResponse200TypeItem",
    "MarginCallEnterTransaction",
    "MarginCallEnterTransactionType",
    "MarginCallExitTransaction",
    "MarginCallExitTransactionType",
    "MarginCallExtendTransaction",
    "MarginCallExtendTransactionType",
    "MarketIfTouchedOrder",
    "MarketIfTouchedOrderPositionFill",
    "MarketIfTouchedOrderReason",
    "MarketIfTouchedOrderRejectTransaction",
    "MarketIfTouchedOrderRejectTransactionPositionFill",
    "MarketIfTouchedOrderRejectTransactionReason",
    "MarketIfTouchedOrderRejectTransactionRejectReason",
    "MarketIfTouchedOrderRejectTransactionTimeInForce",
    "MarketIfTouchedOrderRejectTransactionTriggerCondition",
    "MarketIfTouchedOrderRejectTransactionType",
    "MarketIfTouchedOrderRequest",
    "MarketIfTouchedOrderRequestPositionFill",
    "MarketIfTouchedOrderRequestTimeInForce",
    "MarketIfTouchedOrderRequestTriggerCondition",
    "MarketIfTouchedOrderRequestType",
    "MarketIfTouchedOrderState",
    "MarketIfTouchedOrderTimeInForce",
    "MarketIfTouchedOrderTransaction",
    "MarketIfTouchedOrderTransactionPositionFill",
    "MarketIfTouchedOrderTransactionReason",
    "MarketIfTouchedOrderTransactionTimeInForce",
    "MarketIfTouchedOrderTransactionTriggerCondition",
    "MarketIfTouchedOrderTransactionType",
    "MarketIfTouchedOrderTriggerCondition",
    "MarketIfTouchedOrderType",
    "MarketOrder",
    "MarketOrderDelayedTradeClose",
    "MarketOrderMarginCloseout",
    "MarketOrderMarginCloseoutReason",
    "MarketOrderPositionCloseout",
    "MarketOrderPositionFill",
    "MarketOrderReason",
    "MarketOrderRejectTransaction",
    "MarketOrderRejectTransactionPositionFill",
    "MarketOrderRejectTransactionReason",
    "MarketOrderRejectTransactionRejectReason",
    "MarketOrderRejectTransactionTimeInForce",
    "MarketOrderRejectTransactionType",
    "MarketOrderRequest",
    "MarketOrderRequestPositionFill",
    "MarketOrderRequestTimeInForce",
    "MarketOrderRequestType",
    "MarketOrderState",
    "MarketOrderTimeInForce",
    "MarketOrderTradeClose",
    "MarketOrderTransaction",
    "MarketOrderTransactionPositionFill",
    "MarketOrderTransactionReason",
    "MarketOrderTransactionTimeInForce",
    "MarketOrderTransactionType",
    "MarketOrderType",
    "MT4TransactionHeartbeat",
    "OpenTradeFinancing",
    "Order",
    "OrderBook",
    "OrderBookBucket",
    "OrderCancelReason",
    "OrderCancelRejectTransaction",
    "OrderCancelRejectTransactionRejectReason",
    "OrderCancelRejectTransactionType",
    "OrderCancelTransaction",
    "OrderCancelTransactionReason",
    "OrderCancelTransactionType",
    "OrderClientExtensionsModifyRejectTransaction",
    "OrderClientExtensionsModifyRejectTransactionRejectReason",
    "OrderClientExtensionsModifyRejectTransactionType",
    "OrderClientExtensionsModifyTransaction",
    "OrderClientExtensionsModifyTransactionType",
    "OrderFillReason",
    "OrderFillTransaction",
    "OrderFillTransactionReason",
    "OrderFillTransactionType",
    "OrderIdentifier",
    "OrderPositionFill",
    "OrderRequest",
    "OrderState",
    "OrderStateFilter",
    "OrderTriggerCondition",
    "OrderType",
    "Position",
    "PositionAggregationMode",
    "PositionBook",
    "PositionBookBucket",
    "PositionFinancing",
    "PositionSide",
    "Price",
    "PriceBucket",
    "PriceStatus",
    "PricingHeartbeat",
    "QuoteHomeConversionFactors",
    "ReopenTransaction",
    "ReopenTransactionType",
    "ReplaceOrderBody",
    "ReplaceOrderResponse201",
    "ReplaceOrderResponse400",
    "ReplaceOrderResponse404",
    "ResetResettablePLTransaction",
    "ResetResettablePLTransactionType",
    "SetOrderClientExtensionsBody",
    "SetOrderClientExtensionsResponse200",
    "SetOrderClientExtensionsResponse400",
    "SetOrderClientExtensionsResponse404",
    "SetTradeClientExtensionsBody",
    "SetTradeClientExtensionsResponse200",
    "SetTradeClientExtensionsResponse400",
    "SetTradeClientExtensionsResponse404",
    "SetTradeDependentOrdersBody",
    "SetTradeDependentOrdersResponse200",
    "SetTradeDependentOrdersResponse400",
    "StopLossDetails",
    "StopLossDetailsTimeInForce",
    "StopLossOrder",
    "StopLossOrderReason",
    "StopLossOrderRejectTransaction",
    "StopLossOrderRejectTransactionReason",
    "StopLossOrderRejectTransactionRejectReason",
    "StopLossOrderRejectTransactionTimeInForce",
    "StopLossOrderRejectTransactionTriggerCondition",
    "StopLossOrderRejectTransactionType",
    "StopLossOrderRequest",
    "StopLossOrderRequestTimeInForce",
    "StopLossOrderRequestTriggerCondition",
    "StopLossOrderRequestType",
    "StopLossOrderState",
    "StopLossOrderTimeInForce",
    "StopLossOrderTransaction",
    "StopLossOrderTransactionReason",
    "StopLossOrderTransactionTimeInForce",
    "StopLossOrderTransactionTriggerCondition",
    "StopLossOrderTransactionType",
    "StopLossOrderTriggerCondition",
    "StopLossOrderType",
    "StopOrder",
    "StopOrderPositionFill",
    "StopOrderReason",
    "StopOrderRejectTransaction",
    "StopOrderRejectTransactionPositionFill",
    "StopOrderRejectTransactionReason",
    "StopOrderRejectTransactionRejectReason",
    "StopOrderRejectTransactionTimeInForce",
    "StopOrderRejectTransactionTriggerCondition",
    "StopOrderRejectTransactionType",
    "StopOrderRequest",
    "StopOrderRequestPositionFill",
    "StopOrderRequestTimeInForce",
    "StopOrderRequestTriggerCondition",
    "StopOrderRequestType",
    "StopOrderState",
    "StopOrderTimeInForce",
    "StopOrderTransaction",
    "StopOrderTransactionPositionFill",
    "StopOrderTransactionReason",
    "StopOrderTransactionTimeInForce",
    "StopOrderTransactionTriggerCondition",
    "StopOrderTransactionType",
    "StopOrderTriggerCondition",
    "StopOrderType",
    "StreamPricingResponse200",
    "StreamTransactionsResponse200",
    "TakeProfitDetails",
    "TakeProfitDetailsTimeInForce",
    "TakeProfitOrder",
    "TakeProfitOrderReason",
    "TakeProfitOrderRejectTransaction",
    "TakeProfitOrderRejectTransactionReason",
    "TakeProfitOrderRejectTransactionRejectReason",
    "TakeProfitOrderRejectTransactionTimeInForce",
    "TakeProfitOrderRejectTransactionTriggerCondition",
    "TakeProfitOrderRejectTransactionType",
    "TakeProfitOrderRequest",
    "TakeProfitOrderRequestTimeInForce",
    "TakeProfitOrderRequestTriggerCondition",
    "TakeProfitOrderRequestType",
    "TakeProfitOrderState",
    "TakeProfitOrderTimeInForce",
    "TakeProfitOrderTransaction",
    "TakeProfitOrderTransactionReason",
    "TakeProfitOrderTransactionTimeInForce",
    "TakeProfitOrderTransactionTriggerCondition",
    "TakeProfitOrderTransactionType",
    "TakeProfitOrderTriggerCondition",
    "TakeProfitOrderType",
    "TimeInForce",
    "Trade",
    "TradeClientExtensionsModifyRejectTransaction",
    "TradeClientExtensionsModifyRejectTransactionRejectReason",
    "TradeClientExtensionsModifyRejectTransactionType",
    "TradeClientExtensionsModifyTransaction",
    "TradeClientExtensionsModifyTransactionType",
    "TradeOpen",
    "TradePL",
    "TradeReduce",
    "TradeState",
    "TradeStateFilter",
    "TradeSummary",
    "TradeSummaryState",
    "TrailingStopLossDetails",
    "TrailingStopLossDetailsTimeInForce",
    "TrailingStopLossOrder",
    "TrailingStopLossOrderReason",
    "TrailingStopLossOrderRejectTransaction",
    "TrailingStopLossOrderRejectTransactionReason",
    "TrailingStopLossOrderRejectTransactionRejectReason",
    "TrailingStopLossOrderRejectTransactionTimeInForce",
    "TrailingStopLossOrderRejectTransactionTriggerCondition",
    "TrailingStopLossOrderRejectTransactionType",
    "TrailingStopLossOrderRequest",
    "TrailingStopLossOrderRequestTimeInForce",
    "TrailingStopLossOrderRequestTriggerCondition",
    "TrailingStopLossOrderRequestType",
    "TrailingStopLossOrderState",
    "TrailingStopLossOrderTimeInForce",
    "TrailingStopLossOrderTransaction",
    "TrailingStopLossOrderTransactionReason",
    "TrailingStopLossOrderTransactionTimeInForce",
    "TrailingStopLossOrderTransactionTriggerCondition",
    "TrailingStopLossOrderTransactionType",
    "TrailingStopLossOrderTriggerCondition",
    "TrailingStopLossOrderType",
    "Transaction",
    "TransactionFilter",
    "TransactionHeartbeat",
    "TransactionRejectReason",
    "TransactionType",
    "TransferFundsRejectTransaction",
    "TransferFundsRejectTransactionFundingReason",
    "TransferFundsRejectTransactionRejectReason",
    "TransferFundsRejectTransactionType",
    "TransferFundsTransaction",
    "TransferFundsTransactionFundingReason",
    "TransferFundsTransactionType",
    "UnitsAvailable",
    "UnitsAvailableDetails",
    "UserInfo",
    "UserInfoExternal",
    "WeeklyAlignment",
)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .order_cancel_transaction import OrderCancelTransaction
from .order_fill_transaction import OrderFillTransaction
from .stop_loss_order_transaction import StopLossOrderTransaction
from .take_profit_order_transaction import TakeProfitOrderTransaction
from .trailing_stop_loss_order_transaction import TrailingStopLossOrderTransaction
from typing import List, Optional, Type, TypeVar, cast

T = TypeVar("T", bound="SetTradeDependentOrdersResponse200")


@dataclasses.dataclass
class SetTradeDependentOrdersResponse200:
    """Attributes:
    take_profit_order_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction
        represents the cancellation of an Order in the client's Account.
    take_profit_order_transaction (Optional[TakeProfitOrderTransaction]): A TakeProfitOrderTransaction
        represents the creation of a TakeProfit Order in the user's Account.
    take_profit_order_fill_transaction (Optional[OrderFillTransaction]): An OrderFillTransaction represents the
        filling of an Order in the client's Account.
    take_profit_order_created_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction
        represents the cancellation of an Order in the client's Account.
    stop_loss_order_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction represents
        the cancellation of an Order in the client's Account.
    stop_loss_order_transaction (Optional[StopLossOrderTransaction]): A StopLossOrderTransaction represents the
        creation of a StopLoss Order in the user's Account.
    stop_loss_order_fill_transaction (Optional[OrderFillTransaction]): An OrderFillTransaction represents the
        filling of an Order in the client's Account.
    stop_loss_order_created_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction
        represents the cancellation of an Order in the client's Account.
    trailing_stop_loss_order_cancel_transaction (Optional[OrderCancelTransaction]): An OrderCancelTransaction
        represents the cancellation of an Order in the client's Account.
    trailing_stop_loss_order_transaction (Optional[TrailingStopLossOrderTransaction]): A
        TrailingStopLossOrderTransaction represents the creation of a TrailingStopLoss Order in the user's Account.
    related_transaction_i_ds (Optional[List[str]]): The IDs of all Transactions that were created while
        satisfying the request.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    take_profit_order_cancel_transaction: Optional["OrderCancelTransaction"]
    take_profit_order_transaction: Optional["TakeProfitOrderTransaction"]
    take_profit_order_fill_transaction: Optional["OrderFillTransaction"]
    take_profit_order_created_cancel_transaction: Optional["OrderCancelTransaction"]
    stop_loss_order_cancel_transaction: Optional["OrderCancelTransaction"]
    stop_loss_order_transaction: Optional["StopLossOrderTransaction"]
    stop_loss_order_fill_transaction: Optional["OrderFillTransaction"]
    stop_loss_order_created_cancel_transaction: Optional["OrderCancelTransaction"]
    trailing_stop_loss_order_cancel_transaction: Optional["OrderCancelTransaction"]
    trailing_stop_loss_order_transaction: Optional["TrailingStopLossOrderTransaction"]
    related_transaction_i_ds: Optional[List[str]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .trailing_stop_loss_order_transaction import (
            TrailingStopLossOrderTransaction,
        )
        from .stop_loss_order_transaction import StopLossOrderTransaction
        from .take_profit_order_transaction import TakeProfitOrderTransaction
        from .order_cancel_transaction import OrderCancelTransaction
        from .order_fill_transaction import OrderFillTransaction

        d = src_dict.copy()
        _take_profit_order_cancel_transaction = d.pop(
            "takeProfitOrderCancelTransaction", None
        )
        take_profit_order_cancel_transaction: Optional[OrderCancelTransaction]
        if _take_profit_order_cancel_transaction is None:
            take_profit_order_cancel_transaction = None
        else:
            take_profit_order_cancel_transaction = OrderCancelTransaction.from_dict(
                _take_profit_order_cancel_transaction
            )
        _take_profit_order_transaction = d.pop("takeProfitOrderTransaction", None)
        take_profit_order_transaction: Optional[TakeProfitOrderTransaction]
        if _take_profit_order_transaction is None:
            take_profit_order_transaction = None
        else:
            take_profit_order_transaction = TakeProfitOrderTransaction.from_dict(
                _take_profit_order_transaction
            )
        _take_profit_order_fill_transaction = d.pop(
            "takeProfitOrderFillTransaction", None
        )
        take_profit_order_fill_transaction: Optional[OrderFillTransaction]
        if _take_profit_order_fill_transaction is None:
            take_profit_order_fill_transaction = None
        else:
            take_profit_order_fill_transaction = OrderFillTransaction.from_dict(
                _take_profit_order_fill_transaction
            )
        _take_profit_order_created_cancel_transaction = d.pop(
            "takeProfitOrderCreatedCancelTransaction", None
        )
        take_profit_order_created_cancel_transaction: Optional[OrderCancelTransaction]
        if _take_profit_order_created_cancel_transaction is None:
            take_profit_order_created_cancel_transaction = None
        else:
            take_profit_order_created_cancel_transaction = (
                OrderCancelTransaction.from_dict(
                    _take_profit_order_created_cancel_transaction
                )
            )
        _stop_loss_order_cancel_transaction = d.pop(
            "stopLossOrderCancelTransaction", None
        )
        stop_loss_order_cancel_transaction: Optional[OrderCancelTransaction]
        if _stop_loss_order_cancel_transaction is None:
            stop_loss_order_cancel_transaction = None
        else:
            stop_loss_order_cancel_transaction = OrderCancelTransaction.from_dict(
                _stop_loss_order_cancel_transaction
            )
        _stop_loss_order_transaction = d.pop("stopLossOrderTransaction", None)
        stop_loss_order_transaction: Optional[StopLossOrderTransaction]
        if _stop_loss_order_transaction is None:
            stop_loss_order_transaction = None
        else:
            stop_loss_order_transaction = StopLossOrderTransaction.from_dict(
                _stop_loss_order_transaction
            )
        _stop_loss_order_fill_transaction = d.pop("stopLossOrderFillTransaction", None)
        stop_loss_order_fill_transaction: Optional[OrderFillTransaction]
        if _stop_loss_order_fill_transaction is None:
            stop_loss_order_fill_transaction = None
        else:
            stop_loss_order_fill_transaction = OrderFillTransaction.from_dict(
                _stop_loss_order_fill_transaction
            )
        _stop_loss_order_created_cancel_transaction = d.pop(
            "stopLossOrderCreatedCancelTransaction", None
        )
        stop_loss_order_created_cancel_transaction: Optional[OrderCancelTransaction]
        if _stop_loss_order_created_cancel_transaction is None:
            stop_loss_order_created_cancel_transaction = None
        else:
            stop_loss_order_created_cancel_transaction = (
                OrderCancelTransaction.from_dict(
                    _stop_loss_order_created_cancel_transaction
                )
            )
        _trailing_stop_loss_order_cancel_transaction = d.pop(
            "trailingStopLossOrderCancelTransaction", None
        )
        trailing_stop_loss_order_cancel_transaction: Optional[OrderCancelTransaction]
        if _trailing_stop_loss_order_cancel_transaction is None:
            trailing_stop_loss_order_cancel_transaction = None
        else:
            trailing_stop_loss_order_cancel_transaction = (
                OrderCancelTransaction.from_dict(
                    _trailing_stop_loss_order_cancel_transaction
                )
            )
        _trailing_stop_loss_order_transaction = d.pop(
            "trailingStopLossOrderTransaction", None
        )
        trailing_stop_loss_order_transaction: Optional[TrailingStopLossOrderTransaction]
        if _trailing_stop_loss_order_transaction is None:
            trailing_stop_loss_order_transaction = None
        else:
            trailing_stop_loss_order_transaction = (
                TrailingStopLossOrderTransaction.from_dict(
                    _trailing_stop_loss_order_transaction
                )
            )
        related_transaction_i_ds = cast(List[str], d.pop("relatedTransactionIDs", None))
        last_transaction_id = d.pop("lastTransactionID", None)
        set_trade_dependent_orders_response_200 = cls(
            take_profit_order_cancel_transaction=take_profit_order_cancel_transaction,
            take_profit_order_transaction=take_profit_order_transaction,
            take_profit_order_fill_transaction=take_profit_order_fill_transaction,
            take_profit_order_created_cancel_transaction=take_profit_order_created_cancel_transaction,
            stop_loss_order_cancel_transaction=stop_loss_order_cancel_transaction,
            stop_loss_order_transaction=stop_loss_order_transaction,
            stop_loss_order_fill_transaction=stop_loss_order_fill_transaction,
            stop_loss_order_created_cancel_transaction=stop_loss_order_created_cancel_transaction,
            trailing_stop_loss_order_cancel_transaction=trailing_stop_loss_order_cancel_transaction,
            trailing_stop_loss_order_transaction=trailing_stop_loss_order_transaction,
            related_transaction_i_ds=related_transaction_i_ds,
            last_transaction_id=last_transaction_id,
        )
        set_trade_dependent_orders_response_200.additional_properties = d
        return set_trade_dependent_orders_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .client_extensions import ClientExtensions
from .market_order_delayed_trade_close import MarketOrderDelayedTradeClose
from .market_order_margin_closeout import MarketOrderMarginCloseout
from .market_order_position_closeout import MarketOrderPositionCloseout
from .market_order_reject_transaction_position_fill import (
    MarketOrderRejectTransactionPositionFill,
)
from .market_order_reject_transaction_position_fill import (
    check_market_order_reject_transaction_position_fill,
)
from .market_order_reject_transaction_reason import MarketOrderRejectTransactionReason
from .market_order_reject_transaction_reason import (
    check_market_order_reject_transaction_reason,
)
from .market_order_reject_transaction_reject_reason import (
    MarketOrderRejectTransactionRejectReason,
)
from .market_order_reject_transaction_reject_reason import (
    check_market_order_reject_transaction_reject_reason,
)
from .market_order_reject_transaction_time_in_force import (
    MarketOrderRejectTransactionTimeInForce,
)
from .market_order_reject_transaction_time_in_force import (
    check_market_order_reject_transaction_time_in_force,
)
from .market_order_reject_transaction_type import MarketOrderRejectTransactionType
from .market_order_reject_transaction_type import (
    check_market_order_reject_transaction_type,
)
from .market_order_trade_close import MarketOrderTradeClose
from .stop_loss_details import StopLossDetails
from .take_profit_details import TakeProfitDetails
from .trailing_stop_loss_details import TrailingStopLossDetails
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="MarketOrderRejectTransaction")


@dataclasses.dataclass
class MarketOrderRejectTransaction:
    """A MarketOrderRejectTransaction represents the rejection of the creation of a Market Order.

    Attributes:
        id (Optional[str]): The Transaction's Identifier.
        time (Optional[str]): The date/time when the Transaction was created.
        user_id (Optional[int]): The ID of the user that initiated the creation of the Transaction.
        account_id (Optional[str]): The ID of the Account the Transaction was created for.
        batch_id (Optional[str]): The ID of the "batch" that the Transaction belongs to. Transactions in the same
            batch are applied to the Account simultaneously.
        request_id (Optional[str]): The Request ID of the request which generated the transaction.
        type (Optional[MarketOrderRejectTransactionType]): The Type of the Transaction. Always set to
            "MARKET_ORDER_REJECT" in a MarketOrderRejectTransaction.
        instrument (Optional[str]): The Market Order's Instrument.
        units (Optional[str]): The quantity requested to be filled by the Market Order. A posititive number of units
            results in a long Order, and a negative number of units results in a short Order.
        time_in_force (Optional[MarketOrderRejectTransactionTimeInForce]): The time-in-force requested for the
            Market Order. Restricted to FOK or IOC for a MarketOrder.
        price_bound (Optional[str]): The worst price that the client is willing to have the Market Order filled at.
        position_fill (Optional[MarketOrderRejectTransactionPositionFill]): Specification of how Positions in the
            Account are modified when the Order is filled.
        trade_close (Optional[MarketOrderTradeClose]): A MarketOrderTradeClose specifies the extensions to a Market
            Order that has been created specifically to close a Trade.
        long_position_closeout (Optional[MarketOrderPositionCloseout]): A MarketOrderPositionCloseout specifies the
            extensions to a Market Order when it has been created to closeout a specific Position.
        short_position_closeout (Optional[MarketOrderPositionCloseout]): A MarketOrderPositionCloseout specifies the
            extensions to a Market Order when it has been created to closeout a specific Position.
        margin_closeout (Optional[MarketOrderMarginCloseout]): Details for the Market Order extensions specific to a
            Market Order placed that is part of a Market Order Margin Closeout in a client's account
        delayed_trade_close (Optional[MarketOrderDelayedTradeClose]): Details for the Market Order extensions
            specific to a Market Order placed with the intent of fully closing a specific open trade that should have
            already been closed but wasn't due to halted market conditions
        reason (Optional[MarketOrderRejectTransactionReason]): The reason that the Market Order was created
        client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        take_profit_on_fill (Optional[TakeProfitDetails]): TakeProfitDetails specifies the details of a Take Profit
            Order to be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring
            a Take Profit, or when a Trade's dependent Take Profit Order is modified directly through the Trade.
        stop_loss_on_fill (Optional[StopLossDetails]): StopLossDetails specifies the details of a Stop Loss Order to
            be created on behalf of a client. This may happen when an Order is filled that opens a Trade requiring a Stop
            Loss, or when a Trade's dependent Stop Loss Order is modified directly through the Trade.
        trailing_stop_loss_on_fill (Optional[TrailingStopLossDetails]): TrailingStopLossDetails specifies the
            details of a Trailing Stop Loss Order to be created on behalf of a client. This may happen when an Order is
            filled that opens a Trade requiring a Trailing Stop Loss, or when a Trade's dependent Trailing Stop Loss Order
            is modified directly through the Trade.
        trade_client_extensions (Optional[ClientExtensions]): A ClientExtensions object allows a client to attach a
            clientID, tag and comment to Orders and Trades in their Account.  Do not set, modify, or delete this field if
            your account is associated with MT4.
        reject_reason (Optional[MarketOrderRejectTransactionRejectReason]): The reason that the Reject Transaction
            was created"""

    id: Optional[str]
    time: Optional[str]
    user_id: Optional[int]
    account_id: Optional[str]
    batch_id: Optional[str]
    request_id: Optional[str]
    type: Optional[MarketOrderRejectTransactionType]
    instrument: Optional[str]
    units: Optional[str]
    time_in_force: Optional[MarketOrderRejectTransactionTimeInForce]
    price_bound: Optional[str]
    position_fill: Optional[MarketOrderRejectTransactionPositionFill]
    trade_close: Optional["MarketOrderTradeClose"]
    long_position_closeout: Optional["MarketOrderPositionCloseout"]
    short_position_closeout: Optional["MarketOrderPositionCloseout"]
    margin_closeout: Optional["MarketOrderMarginCloseout"]
    delayed_trade_close: Optional["MarketOrderDelayedTradeClose"]
    reason: Optional[MarketOrderRejectTransactionReason]
    client_extensions: Optional["ClientExtensions"]
    take_profit_on_fill: Optional["TakeProfitDetails"]
    stop_loss_on_fill: Optional["StopLossDetails"]
    trailing_stop_loss_on_fill: Optional["TrailingStopLossDetails"]
    trade_client_extensions: Optional["ClientExtensions"]
    reject_reason: Optional[MarketOrderRejectTransactionRejectReason]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .market_order_position_closeout import MarketOrderPositionCloseout
        from .client_extensions import ClientExtensions
        from .take_profit_details import TakeProfitDetails
        from .market_order_margin_closeout import MarketOrderMarginCloseout
        from .market_order_delayed_trade_close import MarketOrderDelayedTradeClose
        from .trailing_stop_loss_details import TrailingStopLossDetails
        from .stop_loss_details import StopLossDetails
        from .market_order_trade_close import MarketOrderTradeClose

        d = src_dict.copy()
        id = d.pop("id", None)
        time = d.pop("time", None)
        user_id = d.pop("userID", None)
        account_id = d.pop("accountID", None)
        batch_id = d.pop("batchID", None)
        request_id = d.pop("requestID", None)
        _type = d.pop("type", None)
        type: Optional[MarketOrderRejectTransactionType]
        if _type is None:
            type = None
        else:
            type = check_market_order_reject_transaction_type(_type)
        instrument = d.pop("instrument", None)
        units = d.pop("units", None)
        _time_in_force = d.pop("timeInForce", None)
        time_in_force: Optional[MarketOrderRejectTransactionTimeInForce]
        if _time_in_force is None:
            time_in_force = None
        else:
            time_in_force = check_market_order_reject_transaction_time_in_force(
                _time_in_force
            )
        price_bound = d.pop("priceBound", None)
        _position_fill = d.pop("positionFill", None)
        position_fill: Optional[MarketOrderRejectTransactionPositionFill]
        if _position_fill is None:
            position_fill = None
        else:
            position_fill = check_market_order_reject_transaction_position_fill(
                _position_fill
            )
        _trade_close = d.pop("tradeClose", None)
        trade_close: Optional[MarketOrderTradeClose]
        if _trade_close is None:
            trade_close = None
        else:
            trade_close = MarketOrderTradeClose.from_dict(_trade_close)
        _long_position_closeout = d.pop("longPositionCloseout", None)
        long_position_closeout: Optional[MarketOrderPositionCloseout]
        if _long_position_closeout is None:
            long_position_closeout = None
        else:
            long_position_closeout = MarketOrderPositionCloseout.from_dict(
                _long_position_closeout
            )
        _short_position_closeout = d.pop("shortPositionCloseout", None)
        short_position_closeout: Optional[MarketOrderPositionCloseout]
        if _short_position_closeout is None:
            short_position_closeout = None
        else:
            short_position_closeout = MarketOrderPositionCloseout.from_dict(
                _short_position_closeout
            )
        _margin_closeout = d.pop("marginCloseout", None)
        margin_closeout: Optional[MarketOrderMarginCloseout]
        if _margin_closeout is None:
            margin_closeout = None
        else:
            margin_closeout = MarketOrderMarginCloseout.from_dict(_margin_closeout)
        _delayed_trade_close = d.pop("delayedTradeClose", None)
        delayed_trade_close: Optional[MarketOrderDelayedTradeClose]
        if _delayed_trade_close is None:
            delayed_trade_close = None
        else:
            delayed_trade_close = MarketOrderDelayedTradeClose.from_dict(
                _delayed_trade_close
            )
        _reason = d.pop("reason", None)
        reason: Optional[MarketOrderRejectTransactionReason]
        if _reason is None:
            reason = None
        else:
            reason = check_market_order_reject_transaction_reason(_reason)
        _client_extensions = d.pop("clientExtensions", None)
        client_extensions: Optional[ClientExtensions]
        if _client_extensions is None:
            client_extensions = None
        else:
            client_extensions = ClientExtensions.from_dict(_client_extensions)
        _take_profit_on_fill = d.pop("takeProfitOnFill", None)
        take_profit_on_fill: Optional[TakeProfitDetails]
        if _take_profit_on_fill is None:
            take_profit_on_fill = None
        else:
            take_profit_on_fill = TakeProfitDetails.from_dict(_take_profit_on_fill)
        _stop_loss_on_fill = d.pop("stopLossOnFill", None)
        stop_loss_on_fill: Optional[StopLossDetails]
        if _stop_loss_on_fill is None:
            stop_loss_on_fill = None
        else:
            stop_loss_on_fill = StopLossDetails.from_dict(_stop_loss_on_fill)
        _trailing_stop_loss_on_fill = d.pop("trailingStopLossOnFill", None)
        trailing_stop_loss_on_fill: Optional[TrailingStopLossDetails]
        if _trailing_stop_loss_on_fill is None:
            trailing_stop_loss_on_fill = None
        else:
            trailing_stop_loss_on_fill = TrailingStopLossDetails.from_dict(
                _trailing_stop_loss_on_fill
            )
        _trade_client_extensions = d.pop("tradeClientExtensions", None)
        trade_client_extensions: Optional[ClientExtensions]
        if _trade_client_extensions is None:
            trade_client_extensions = None
        else:
            trade_client_extensions = ClientExtensions.from_dict(
                _trade_client_extensions
            )
        _reject_reason = d.pop("rejectReason", None)
        reject_reason: Optional[MarketOrderRejectTransactionRejectReason]
        if _reject_reason is None:
            reject_reason = None
        else:
            reject_reason = check_market_order_reject_transaction_reject_reason(
                _reject_reason
            )
        market_order_reject_transaction = cls(
            id=id,
            time=time,
            user_id=user_id,
            account_id=account_id,
            batch_id=batch_id,
            request_id=request_id,
            type=type,
            instrument=instrument,
            units=units,
            time_in_force=time_in_force,
            price_bound=price_bound,
            position_fill=position_fill,
            trade_close=trade_close,
            long_position_closeout=long_position_closeout,
            short_position_closeout=short_position_closeout,
            margin_closeout=margin_closeout,
            delayed_trade_close=delayed_trade_close,
            reason=reason,
            client_extensions=client_extensions,
            take_profit_on_fill=take_profit_on_fill,
            stop_loss_on_fill=stop_loss_on_fill,
            trailing_stop_loss_on_fill=trailing_stop_loss_on_fill,
            trade_client_extensions=trade_client_extensions,
            reject_reason=reject_reason,
        )
        market_order_reject_transaction.additional_properties = d
        return market_order_reject_transaction

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .price_bucket import PriceBucket
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="Price")


@dataclasses.dataclass
class Price:
    """The Price representation

    Attributes:
        instrument (Optional[str]): The Price's Instrument.
        tradeable (Optional[bool]): Flag indicating if the Price is tradeable or not
        timestamp (Optional[str]): The date/time when the Price was created.
        base_bid (Optional[str]): The base bid price as calculated by pricing.
        base_ask (Optional[str]): The base ask price as calculated by pricing.
        bids (Optional[List['PriceBucket']]): The list of prices and liquidity available on the Instrument's bid
            side. It is possible for this list to be empty if there is no bid liquidity currently available for the
            Instrument in the Account.
        asks (Optional[List['PriceBucket']]): The list of prices and liquidity available on the Instrument's ask
            side. It is possible for this list to be empty if there is no ask liquidity currently available for the
            Instrument in the Account.
        closeout_bid (Optional[str]): The closeout bid price. This price is used when a bid is required to closeout
            a Position (margin closeout or manual) yet there is no bid liquidity. The closeout bid is never used to open a
            new position.
        closeout_ask (Optional[str]): The closeout ask price. This price is used when an ask is required to closeout
            a Position (margin closeout or manual) yet there is no ask liquidity. The closeout ask is never used to open a
            new position."""

    instrument: Optional[str]
    tradeable: Optional[bool]
    timestamp: Optional[str]
    base_bid: Optional[str]
    base_ask: Optional[str]
    bids: Optional[List["PriceBucket"]]
    asks: Optional[List["PriceBucket"]]
    closeout_bid: Optional[str]
    closeout_ask: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .price_bucket import PriceBucket

        d = src_dict.copy()
        instrument = d.pop("instrument", None)
        tradeable = d.pop("tradeable", None)
        timestamp = d.pop("timestamp", None)
        base_bid = d.pop("baseBid", None)
        base_ask = d.pop("baseAsk", None)
        bids = []
        _bids = d.pop("bids", None)
        for bids_item_data in _bids or []:
            bids_item = PriceBucket.from_dict(bids_item_data)
            bids.append(bids_item)
        asks = []
        _asks = d.pop("asks", None)
        for asks_item_data in _asks or []:
            asks_item = PriceBucket.from_dict(asks_item_data)
            asks.append(asks_item)
        closeout_bid = d.pop("closeoutBid", None)
        closeout_ask = d.pop("closeoutAsk", None)
        price = cls(
            instrument=instrument,
            tradeable=tradeable,
            timestamp=timestamp,
            base_bid=base_bid,
            base_ask=base_ask,
            bids=bids,
            asks=asks,
            closeout_bid=closeout_bid,
            closeout_ask=closeout_ask,
        )
        price.additional_properties = d
        return price

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.account_properties import AccountProperties


T = TypeVar("T", bound="ListAccountsResponse200")


@_attrs_define
class ListAccountsResponse200:
    """
    Attributes:
        accounts (Union[Unset, List['AccountProperties']]): The list of Accounts the client is authorized to access and
            their associated properties.
    """

    accounts: Union[Unset, List["AccountProperties"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        accounts: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.accounts, Unset):
            accounts = []
            for accounts_item_data in self.accounts:
                accounts_item = accounts_item_data.to_dict()
                accounts.append(accounts_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if accounts is not UNSET:
            field_dict["accounts"] = accounts

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.account_properties import AccountProperties

        d = src_dict.copy()
        accounts = []
        _accounts = d.pop("accounts", UNSET)
        for accounts_item_data in _accounts or []:
            accounts_item = AccountProperties.from_dict(accounts_item_data)

            accounts.append(accounts_item)

        list_accounts_response_200 = cls(
            accounts=accounts,
        )

        list_accounts_response_200.additional_properties = d
        return list_accounts_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .position_side import PositionSide
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="Position")


@dataclasses.dataclass
class Position:
    """The specification of a Position within an Account.

    Attributes:
        instrument (Optional[str]): The Position's Instrument.
        pl (Optional[str]): Profit/loss realized by the Position over the lifetime of the Account.
        unrealized_pl (Optional[str]): The unrealized profit/loss of all open Trades that contribute to this
            Position.
        margin_used (Optional[str]): Margin currently used by the Position.
        resettable_pl (Optional[str]): Profit/loss realized by the Position since the Account's resettablePL was
            last reset by the client.
        financing (Optional[str]): The total amount of financing paid/collected for this instrument over the
            lifetime of the Account.
        commission (Optional[str]): The total amount of commission paid for this instrument over the lifetime of the
            Account.
        guaranteed_execution_fees (Optional[str]): The total amount of fees charged over the lifetime of the Account
            for the execution of guaranteed Stop Loss Orders for this instrument.
        long (Optional[PositionSide]): The representation of a Position for a single direction (long or short).
        short (Optional[PositionSide]): The representation of a Position for a single direction (long or short)."""

    instrument: Optional[str]
    pl: Optional[str]
    unrealized_pl: Optional[str]
    margin_used: Optional[str]
    resettable_pl: Optional[str]
    financing: Optional[str]
    commission: Optional[str]
    guaranteed_execution_fees: Optional[str]
    long: Optional["PositionSide"]
    short: Optional["PositionSide"]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .position_side import PositionSide

        d = src_dict.copy()
        instrument = d.pop("instrument", None)
        pl = d.pop("pl", None)
        unrealized_pl = d.pop("unrealizedPL", None)
        margin_used = d.pop("marginUsed", None)
        resettable_pl = d.pop("resettablePL", None)
        financing = d.pop("financing", None)
        commission = d.pop("commission", None)
        guaranteed_execution_fees = d.pop("guaranteedExecutionFees", None)
        _long = d.pop("long", None)
        long: Optional[PositionSide]
        if _long is None:
            long = None
        else:
            long = PositionSide.from_dict(_long)
        _short = d.pop("short", None)
        short: Optional[PositionSide]
        if _short is None:
            short = None
        else:
            short = PositionSide.from_dict(_short)
        position = cls(
            instrument=instrument,
            pl=pl,
            unrealized_pl=unrealized_pl,
            margin_used=margin_used,
            resettable_pl=resettable_pl,
            financing=financing,
            commission=commission,
            guaranteed_execution_fees=guaranteed_execution_fees,
            long=long,
            short=short,
        )
        position.additional_properties = d
        return position

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .transaction import Transaction
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="GetTransactionRangeResponse200")


@dataclasses.dataclass
class GetTransactionRangeResponse200:
    """Attributes:
    transactions (Optional[List['Transaction']]): The list of Transactions that satisfy the request.
    last_transaction_id (Optional[str]): The ID of the most recent Transaction created for the Account"""

    transactions: Optional[List["Transaction"]]
    last_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .transaction import Transaction

        d = src_dict.copy()
        transactions = []
        _transactions = d.pop("transactions", None)
        for transactions_item_data in _transactions or []:
            transactions_item = Transaction.from_dict(transactions_item_data)
            transactions.append(transactions_item)
        last_transaction_id = d.pop("lastTransactionID", None)
        get_transaction_range_response_200 = cls(
            transactions=transactions, last_transaction_id=last_transaction_id
        )
        get_transaction_range_response_200.additional_properties = d
        return get_transaction_range_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="CandlestickData")


@dataclasses.dataclass
class CandlestickData:
    """The price data (open, high, low, close) for the Candlestick representation.

    Attributes:
        o (Optional[str]): The first (open) price in the time-range represented by the candlestick.
        h (Optional[str]): The highest price in the time-range represented by the candlestick.
        l (Optional[str]): The lowest price in the time-range represented by the candlestick.
        c (Optional[str]): The last (closing) price in the time-range represented by the candlestick."""

    o: Optional[str]
    h: Optional[str]
    l: Optional[str]
    c: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        o = d.pop("o", None)
        h = d.pop("h", None)
        l = d.pop("l", None)
        c = d.pop("c", None)
        candlestick_data = cls(o=o, h=h, l=l, c=c)
        candlestick_data.additional_properties = d
        return candlestick_data

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from .account_properties import AccountProperties
from typing import List, Optional, Type, TypeVar

T = TypeVar("T", bound="ListAccountsResponse200")


@dataclasses.dataclass
class ListAccountsResponse200:
    """Attributes:
    accounts (Optional[List['AccountProperties']]): The list of Accounts the client is authorized to access and
        their associated properties."""

    accounts: Optional[List["AccountProperties"]]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from .account_properties import AccountProperties

        d = src_dict.copy()
        accounts = []
        _accounts = d.pop("accounts", None)
        for accounts_item_data in _accounts or []:
            accounts_item = AccountProperties.from_dict(accounts_item_data)
            accounts.append(accounts_item)
        list_accounts_response_200 = cls(accounts=accounts)
        list_accounts_response_200.additional_properties = d
        return list_accounts_response_200

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="PositionBookBucket")


@dataclasses.dataclass
class PositionBookBucket:
    """The position book data for a partition of the instrument's prices.

    Attributes:
        price (Optional[str]): The lowest price (inclusive) covered by the bucket. The bucket covers the price range
            from the price to price + the position book's bucketWidth.
        long_count_percent (Optional[str]): The percentage of the total number of positions represented by the long
            positions found in this bucket.
        short_count_percent (Optional[str]): The percentage of the total number of positions represented by the
            short positions found in this bucket."""

    price: Optional[str]
    long_count_percent: Optional[str]
    short_count_percent: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        price = d.pop("price", None)
        long_count_percent = d.pop("longCountPercent", None)
        short_count_percent = d.pop("shortCountPercent", None)
        position_book_bucket = cls(
            price=price,
            long_count_percent=long_count_percent,
            short_count_percent=short_count_percent,
        )
        position_book_bucket.additional_properties = d
        return position_book_bucket

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)



from typing import Literal, Set, cast

FixedPriceOrderPositionFill = Literal[
    "DEFAULT", "OPEN_ONLY", "REDUCE_FIRST", "REDUCE_ONLY"
]

FIXED_PRICE_ORDER_POSITION_FILL_VALUES: Set[FixedPriceOrderPositionFill] = {
    "DEFAULT",
    "OPEN_ONLY",
    "REDUCE_FIRST",
    "REDUCE_ONLY",
}


def check_fixed_price_order_position_fill(value: str) -> FixedPriceOrderPositionFill:
    if value in FIXED_PRICE_ORDER_POSITION_FILL_VALUES:
        return cast(FixedPriceOrderPositionFill, value)
    raise TypeError(
        f"Unexpected value {value!r}. Expected one of {FIXED_PRICE_ORDER_POSITION_FILL_VALUES!r}"
    )



from __future__ import annotations
from typing import Dict, Any
import dataclasses
from typing import Optional, Type, TypeVar

T = TypeVar("T", bound="MarketOrderDelayedTradeClose")


@dataclasses.dataclass
class MarketOrderDelayedTradeClose:
    """Details for the Market Order extensions specific to a Market Order placed with the intent of fully closing a
    specific open trade that should have already been closed but wasn't due to halted market conditions

        Attributes:
            trade_id (Optional[str]): The ID of the Trade being closed
            client_trade_id (Optional[str]): The Client ID of the Trade being closed
            source_transaction_id (Optional[str]): The Transaction ID of the DelayedTradeClosure transaction to which
                this Delayed Trade Close belongs to"""

    trade_id: Optional[str]
    client_trade_id: Optional[str]
    source_transaction_id: Optional[str]

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        trade_id = d.pop("tradeID", None)
        client_trade_id = d.pop("clientTradeID", None)
        source_transaction_id = d.pop("sourceTransactionID", None)
        market_order_delayed_trade_close = cls(
            trade_id=trade_id,
            client_trade_id=client_trade_id,
            source_transaction_id=source_transaction_id,
        )
        market_order_delayed_trade_close.additional_properties = d
        return market_order_delayed_trade_close

    def to_dict(self) -> Dict[str, Any]:
        """Convert the dataclass instance to a dictionary."""
        return dataclasses.asdict(self)
